/*
 * instr.c
 *
 *  Created on: Oct 8, 2015
 *      Author: heso
 */

#include "instr.h" //glob.h, condReg.h included
#include "ioMap.h"
#include "hex.h"
#include "regFile.h"
#include "vcd.h"
#include "report.h"
//#include "condReg.h"

//==============================================================
//IO
void in(unsigned int reg)
{
	reportInstr(I_IN);
	unsigned int val;
	val = ioRead();
	regWrite(reg, val);
	vcdWaitCycle();
}

void out(io_t padCtrl, unsigned int reg)
{
	reportInstr(I_OUT);
	unsigned int val;
	val = regRead(reg);
	ioWrite(padCtrl, val);
	vcdWaitCycle();
}

void outi(io_t padCtrl, unsigned int imm)
{
	reportInstr(I_OUTI);
	ioWrite(padCtrl, imm);
	vcdWaitCycle();
}

//==============================================================
//Logic
void andi(unsigned int regd, unsigned int imm)
{
	reportInstr(I_ANDI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald & imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void and(unsigned int regd, unsigned int rega)
{
	reportInstr(I_AND);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald & vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void ori(unsigned int regd, unsigned int imm)
{
	reportInstr(I_ORI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald | imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void or(unsigned int regd, unsigned int rega)
{
	reportInstr(I_OR);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald | vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void xor(unsigned int regd, unsigned int rega)
{
	reportInstr(I_XOR);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald ^ vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void xori(unsigned int regd, unsigned int imm)
{
	reportInstr(I_XORI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald ^ imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}

//==============================================================
//Shift
void lsr(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LSR);
	unsigned int vald;
	vald = regRead(regd);
	vald >>= imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void lsl(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LSL);
	unsigned int vald;
	vald = regRead(regd);
	vald <<= imm;
	vald = vald & 0xFF; //Mask 9th bit.
	regWrite(regd, vald);
	vcdWaitCycle();
}

//==============================================================
//Load
void ldi(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LDI);
	regWrite(regd, imm);
	vcdWaitCycle();
}

void mov(unsigned int regd, unsigned int rega)
{
	reportInstr(I_MOV);
	unsigned int val;
	val = regRead(rega);
	regWrite(regd, val);
	vcdWaitCycle();
}

//==============================================================
//SOL
void som(io_t padCtrl, unsigned int addr, unsigned int reg)
{
	reportInstr(I_SOM);
	unsigned int val, bus;
	val = regRead(reg);

	bus = regReadTxReg(MSB);
	if (bus)
	{
		bus >>= (7 - addr);
		val = val | bus;
	}
	else
	{
		val = val & ~(1 << addr);
	}
	ioWrite(padCtrl, val);
	regWrite(reg, val);
	vcdWaitCycle();
}

void sol(io_t padCtrl, unsigned int addr, unsigned int reg)
{
	reportInstr(I_SOL);
	unsigned int val, bus;
	val = regRead(reg);

	bus = regReadTxReg(LSB);

	if (bus)
	{
		bus <<= addr;
		val = val | bus;
	}
	else
	{
		val = val & ~(1 << addr);
	}
	ioWrite(padCtrl, val);
	regWrite(reg, val);
	vcdWaitCycle();
}

void sim(unsigned int addr, unsigned int reg)
{
	reportInstr(I_SIM);
	unsigned int bus;
	bus = ioRead();

	regWrite(reg, bus);

	bus <<= 7 - addr; //Shift addr-bit to MSB position
	bus &= 0x80; //Mask

	regWriteRxReg(MSB, bus);
	vcdWaitCycle();
}
void sil(unsigned int addr, unsigned int reg)
{
	reportInstr(I_SIL);
	unsigned int bus;
	bus = ioRead();

	regWrite(reg, bus);

	bus >>= addr; //Shift addr-bit to LSB position
	bus &= 0x01; //Mask

	regWriteRxReg(LSB, bus);
	vcdWaitCycle();
}

//==============================================================
//Skip
void cpse(unsigned int rega, unsigned int boolean)
{
	reportInstr(I_CPSE);
	vcdWaitCycle();
}
void cbse(unsigned int rega, unsigned int bit, unsigned int boolean)
{
	reportInstr(I_CBSE);
	vcdWaitCycle();
}

int cbseJmp(unsigned int rega, unsigned int bit, unsigned int boolean, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CBSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (((val & 1<<bit) && 1) != boolean)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();

	return 0;
}


void cpseLogic(unsigned int rega, unsigned int imm, void (*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2)
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();

	if (val != imm)
	{
		logicFunc(var1, var2);
	}
	else
		nop();

}

void cpsneLogic(unsigned int rega, unsigned int imm, void (*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2)
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();

	if (val == imm)
	{
		logicFunc(var1, var2);
	}
	else
		nop();

}

//Unnecessary to pass jmpFunc if there is only one type of jmp function.
int cpseJmp(unsigned int rega, unsigned int imm, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (val != imm)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();

	return 0;
}

int cpsneJmp(unsigned int rega, unsigned int imm, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (val == imm)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();

	return 0;
}
//==============================================================
//Misc control
void repeat(unsigned int iter, void (*section)())
{
	reportInstr(I_REPEAT);
	int i;
	vcdWaitCycle();
	for (i = 0; i < iter; i++)
	{
		section();
	}
}

void jmp(void (*section)())
{
	reportPipeFlush();
	reportInstr(I_JMP);
	vcdPipeFlush();
	vcdWaitCycle();
	section();
}

void jmpDummy(char* destination)
{
	reportPipeFlush();
	reportInstr(I_JMP);
	vcdPipeFlush();
	vcdWaitCycle();
}
//Set
void timst(unsigned int time)
{
	reportInstr(I_TIMST);
	vcdWaitCycle();
	vcdStartTimer(time);
}

void wfe(void)
{
	reportInstr(I_WFE);
	//Prefetch, put processor in sleep and wait.
	vcdWaitCycle();
	vcdWaitTimer();
}

void nop(void)
{
	reportInstr(I_NOP);
	vcdWaitCycle();
}
//==============================================================

//===============================================================
//=======================CONDITIONALS============================
//===============================================================

void cpiec(condReg_t cReg, unsigned int reg, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		unsigned int val;
		val = regRead(reg);
		if(val == imm)
			condRegWrite(cReg, 1);
		else
			condRegWrite(cReg, 0);
	}
	reportInstr(I_CPI);
	vcdWaitCycle();
}

void cpinec(condReg_t cReg, unsigned int reg, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		unsigned int val;
		val = regRead(reg);
		if(val != imm)
			condRegWrite(cReg, 1);
		else
			condRegWrite(cReg, 0);
	}
	reportInstr(I_CPI);
	vcdWaitCycle();
}

void cpibec(condReg_t cReg, unsigned int reg, unsigned int bit, unsigned int boolean, unsigned int cond)
{
	if (cond)
	{
		unsigned int val;
		val = regRead(reg);
		if(((val & 1<<bit) && 1) == boolean)
			condRegWrite(cReg, 1);
		else
			condRegWrite(cReg, 0);
	}
	reportInstr(I_CPIB);
	vcdWaitCycle();
}

void setc(unsigned int cReg, unsigned int cond)
{
	reportInstr(I_SET);
	vcdWaitCycle();
	if(cond)
	{
		condRegWrite(cReg, 1);
	}
}

void clrc(unsigned int cReg, unsigned int cond)
{
	reportInstr(I_CLR);
	vcdWaitCycle();
	if(cond)
	{
		condRegWrite(cReg, 0);
	}
}

//==============================================================
//IO
void inc(unsigned int reg, unsigned int cond)
{
	if (cond)
	{
		unsigned int val;
		val = ioRead();
		regWrite(reg, val);
	}
	reportInstr(I_IN);
	vcdWaitCycle();
}

void outc(io_t padCtrl, unsigned int reg, unsigned int cond)
{
	if (cond)
	{
		unsigned int val;
		val = regRead(reg);
		ioWrite(padCtrl, val);
	}
	reportInstr(I_OUT);
	vcdWaitCycle();
}

void outic(io_t padCtrl, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		ioWrite(padCtrl, imm);
	}
	reportInstr(I_OUTI);
	vcdWaitCycle();
}

//==============================================================
//Logic
void andic(unsigned int regd, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		unsigned int vald;
		vald = regRead(regd);
		vald = vald & imm;
		regWrite(regd, vald);
	}
	reportInstr(I_ANDI);
	vcdWaitCycle();
}
void andc(unsigned int regd, unsigned int rega, unsigned int cond)
{
	if (cond)
	{
		unsigned int vald, vala;
		vald = regRead(regd);
		vala = regRead(rega);
		vald = vald & vala;
		regWrite(regd, vald);
	}
	reportInstr(I_AND);
	vcdWaitCycle();
}
void oric(unsigned int regd, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		unsigned int vald;
		vald = regRead(regd);
		vald = vald | imm;
		regWrite(regd, vald);
	}
	reportInstr(I_ORI);
	vcdWaitCycle();
}
void orc(unsigned int regd, unsigned int rega, unsigned int cond)
{
	if (cond)
	{
		unsigned int vald, vala;
		vald = regRead(regd);
		vala = regRead(rega);
		vald = vald | vala;
		regWrite(regd, vald);
	}
	reportInstr(I_OR);
	vcdWaitCycle();
}
void xorc(unsigned int regd, unsigned int rega, unsigned int cond)
{
	if (cond)
	{
		unsigned int vald, vala;
		vald = regRead(regd);
		vala = regRead(rega);
		vald = vald ^ vala;
		regWrite(regd, vald);
	}
	reportInstr(I_XOR);
	vcdWaitCycle();
}
void xoric(unsigned int regd, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		unsigned int vald;
		vald = regRead(regd);
		vald = vald ^ imm;
		regWrite(regd, vald);
	}
	reportInstr(I_XORI);
	vcdWaitCycle();
}

//==============================================================
//Shift
void lsrc(unsigned int regd, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		unsigned int vald;
		vald = regRead(regd);
		vald >>= imm;
		regWrite(regd, vald);
	}
	reportInstr(I_LSR);
	vcdWaitCycle();
}
void lslc(unsigned int regd, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		unsigned int vald;
		vald = regRead(regd);
		vald <<= imm;
		vald = vald & 0xFF; //Mask 9th bit.
		regWrite(regd, vald);
	}
	reportInstr(I_LSL);
	vcdWaitCycle();
}

//==============================================================
//Load
void ldic(unsigned int regd, unsigned int imm, unsigned int cond)
{
	if (cond)
	{
		regWrite(regd, imm);
	}
	reportInstr(I_LDI);
	vcdWaitCycle();
}

void movc(unsigned int regd, unsigned int rega, unsigned int cond)
{
	if (cond)
	{
		unsigned int val;
		val = regRead(rega);
		regWrite(regd, val);
	}
	reportInstr(I_MOV);
	vcdWaitCycle();
}

//==============================================================
//SOL
void somc(io_t padCtrl, unsigned int addr, unsigned int reg, unsigned int cond)
{
	if (cond)
	{
		unsigned int val, bus;
		val = regRead(reg);

		bus = regReadTxReg(MSB);
		if (bus)
		{
			bus >>= (7 - addr);
			val = val | bus;
		}
		else
		{
			val = val & ~(1 << addr);
		}
		ioWrite(padCtrl, val);
		regWrite(reg, val);
	}
	reportInstr(I_SOM);
	vcdWaitCycle();
}

void solc(io_t padCtrl, unsigned int addr, unsigned int reg, unsigned int cond)
{
	if (cond)
	{
		unsigned int val, bus;
		val = regRead(reg);

		bus = regReadTxReg(LSB);

		if (bus)
		{
			bus <<= addr;
			val = val | bus;
		}
		else
		{
			val = val & ~(1 << addr);
		}
		ioWrite(padCtrl, val);
		regWrite(reg, val);
	}
	reportInstr(I_SOL);
	vcdWaitCycle();
}

void simc(unsigned int addr, unsigned int reg, unsigned int cond)
{
	if (cond)
	{
		unsigned int bus;
		bus = ioRead();

		regWrite(reg, bus);

		bus <<= 7 - addr; //Shift addr-bit to MSB position
		bus &= 0x80; //Mask

		regWriteRxReg(MSB, bus);
	}
	reportInstr(I_SIM);
	vcdWaitCycle();
}
void silc(unsigned int addr, unsigned int reg, unsigned int cond)
{
	if (cond)
	{
		unsigned int bus;
		bus = ioRead();

		regWrite(reg, bus);

		bus >>= addr; //Shift addr-bit to LSB position
		bus &= 0x01; //Mask

		regWriteRxReg(LSB, bus);
	}
	reportInstr(I_SIL);
	vcdWaitCycle();
}

//==============================================================
//Skip
void cpsec(unsigned int rega, unsigned int boolean, unsigned int cond)
{
	if (cond)
	{
	}
	reportInstr(I_CPSE);
	vcdWaitCycle();
}
void cbsec(unsigned int rega, unsigned int bit, unsigned int boolean, unsigned int cond)
{
	if (cond)
	{
	}
	reportInstr(I_CBSE);
	vcdWaitCycle();
}

int cbseJmpc(unsigned int rega, unsigned int bit, unsigned int boolean, void (*jmpFunc)(), void (*section)(), unsigned int cond)
{
	reportInstr(I_CBSE);
	vcdWaitCycle();
	if (cond)
	{
		unsigned int val;
		val = regRead(rega);

		if (((val & 1 << bit) && 1) != boolean)
		{
			jmpFunc(section);
			return 1;
		}
		else
			nop();

	}
	return 0;
}


void cpseLogicc(unsigned int rega, unsigned int imm, void (*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2, unsigned int cond)
{
	reportInstr(I_CPSE);
	vcdWaitCycle();
	if (cond)
	{

		unsigned int val;
		val = regRead(rega);

		if (val != imm)
		{
			logicFunc(var1, var2);
		}
		else
			nop();

	}
}

void cpsneLogicc(unsigned int rega, unsigned int imm, void (*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2, unsigned int cond)
{
	reportInstr(I_CPSE);
	vcdWaitCycle();
	if (cond)
	{

		unsigned int val;
		val = regRead(rega);

		if (val == imm)
		{
			logicFunc(var1, var2);
		}
		else
			nop();

	}
}

//Unnecessary to pass jmpFunc if there is only one type of jmp function.
void cpsneOutc(unsigned int rega, unsigned int imm, void (*outFunc)(), io_t padCtrl, unsigned int immReg, unsigned int cond)
{
	reportInstr(I_CPSE);
	vcdWaitCycle();
	if (cond)
	{

		unsigned int val;
		val = regRead(rega);

		if (val == imm)
		{
			outFunc(padCtrl, immReg);
		}
		else
			nop();
	}
}

int cpsneJmpc(unsigned int rega, unsigned int imm, void (*jmpFunc)(), void (*section)(), unsigned int cond)
{
	reportInstr(I_CPSE);
	vcdWaitCycle();
	if (cond)
	{

		unsigned int val;
		val = regRead(rega);

		if (val == imm)
		{
			jmpFunc(section);
			return 1;
		}
		else
			nop();
	}
	return 0;
}
//==============================================================
//Misc control
void repeatc(unsigned int iter, void (*section)(), unsigned int cond)
{
	reportInstr(I_REPEAT);
	vcdWaitCycle();
	if (cond)
	{
		int i;
		for (i = 0; i < iter; i++)
		{
			section();
		}
	}
}

int jmpc(void (*section)(), unsigned int cond)
{
	vcdWaitCycle();
	reportInstr(I_JMP);
vcdPipeFlush();
	if (cond)
	{
		reportPipeFlush();
		section();
		return 1;
	}
	return 0;
}

int jmpDummyc(char* destination, unsigned int cond)
{
	reportInstr(I_JMP);
vcdPipeFlush();
	vcdWaitCycle();
	if (cond)
	{
		reportPipeFlush();
		return 1;
	}
	return 0;
}
