/*
 * uart.c
 *
 *  Created on: Oct 15, 2015
 *      Author: heso
 */


#include "uart.h"
#include "instr.h" //Includes glob.h
#include "vcd.h"
#include "regFile.h"
#include "condReg.h"

int i;
/*-----------------------------------------------------------------------------------------
	UART with Repeat and SOL and conditionals
-----------------------------------------------------------------------------------------*/
void uartSetup(void)
{
	condRegReset();
	outi (IE, 1<<RXD);	//Enable input RX = in[0]
	outi (OE_N, 0<<TXD & 0xFF);	//Enable output TX = out[7]
	outi (OUT, 1<<TXD);	//Idle
	timst(UART_CK_TIME);		//Baud rate 250K (500kHz)
}

void uartFull(unsigned int cycles)
{
	for (i=0; i<cycles; i++) //== jmp at the end of every run
	{
		uartRun();
	}
}

void uartRun(void)
{
	uartTxIdle();
}

void uartTxIdle(void)
{
	//T3 == running TX
	//T2 == running txStart

	wfe(); //Posedge

//Check Start
	cpibec(rT3, CTRL_REG, TXSTART, 1, !T3);

//Start bit
	outic(OUT, 0<<TXD, T3); //Start bit
	andic(CTRL_REG, 0xFF ^ 1<<TXSTART, T3); //Clear start

	uartRx();

	if( jmpc(uartTxRunEight, T3) ) //If TX started running
		return;

	jmpDummy("uartTxIdle");
}

void uartTxRunEight(void)
{
	repeat(8, uartTxRunning);
	i += 8; //DEBUG
//Stop bit
	outic(OUT, 1<<TXD, T3);
	clrc(rT3, T3); //Clear run
	jmpDummy("uartTxIdle");
}

void uartRx(void)
{
	//T1 == running RX
	//T0 == rxStart

	wfe(); //Read middle

	if( jmpc(uartRxRunning, T1) )
		return;
	else
	{
		//Check for start bit on RXD if not running
		inc(R1, !T1);
		cpiec(rT1, R1, 0<<RXD, !T1); //Set running if start bit
		ldic(R3, 0x80, T1); //Init counter
	}
}

void uartRxRep(void)
{
	//T1 == running RX
	//T0 == rxStart

	wfe(); //Read middle

	//RunningRep and the code below cannot be run without jump because both set or clear T1.
	//Thus, both will run whenever a change is performed.
	//Adding more compares to alleviate this does not increase performance.
	if( jmpc(uartRxRunningRep, T1) ) //Length 4 +flush
		return;
	else
	{
		//Check for start bit on RXD if not running
		inc(R1, !T1);
		cpiec(rT1, R1, 0<<RXD, !T1); //Set running if start bit
		ldic(R3, 0x80, T1); //Init counter
		nop(); //4
		nop(); //Flush
		nop(); //Flush
	}
}


void uartTxRunning(void)
{
	wfe();
//Set TXD
	solc(OUT, TXD, R0, T3);
	lsrc(TX_REG, 1, T3);
	uartRxRep();
}

void uartRxRunning(void)
{
//Read RXD if running
	lsrc (RX_REG, 1, T1);
	simc (RXD, R0, T1);
	lsrc (R3, 1, T1);
//Check if RX is done
	cpinec(rT1, R3, 0x00, T1);	//Set T1 low if counter is zero
	jmpDummy("uartIdleEnd");
}

void uartRxRunningRep(void)
{
//Read RXD if running
	lsrc (RX_REG, 1, T1);
	simc (RXD, R0, T1);
	lsrc (R3, 1, T1);
//Check if RX is done
	cpinec(rT1, R3, 0x00, T1);	//Set T1 low if counter is zero
}
