/*
 * i2c.c
 *
 *  Created on: Nov 10, 2015
 *      Author: heso
 */

#include "i2c.h"
#include "instr.h" //Includes glob.h
#include "regFile.h"
#include <stdio.h>

int i;

/*
 * TO=Addr
 * T1&&!T0=Write
 * !T1&&!T0=read
 * T2 = continue to stop/rep
 * T3 = stop
 */

void i2cMInit(void)
{
	parallelEx(
			outiPar, OUT, 0x00, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			outiPar, OE_N, 0xFF, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			ldiPar, R2, 0x7F, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			outiPar, IE, 1<<SDA | 1<<SCL, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			timstPar, I2C_CK_TIME, NAN, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
}


void i2cM(int transfers)
{	//Simulator workaround
	i=transfers+1;//Number of transfers + address transfer

	//Start condition
	parallelEx(
			outiPar, OE_N, 0xFF ^ 1<<SDA, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			wfePar, NAN, NAN, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			outiPar, OE_N, 0xFF ^ (1<<SDA | 1<<SCL), NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			setPar, rT0, NAN, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);

	i2cParallel();
}

void i2cParallel(void)
{
	repeat(8, i2cParallelBit);
	i2cParallelAck();

	if(
	parallelEx2Jmp(
			jmp, i2cParallel, !T2,
			jmp, i2cMStop, T2,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			))
	{return;}
}

void i2cParallelBit(void)
{
	/*
	 * ex0 sendAddrBit
	 * ex1 sendBit
	 * ex2 recBit
	 */

	parallelEx(
			wfePar, NAN, NAN, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			andiPar, R2, 0xFF ^ 1<<SCL, NAN, T0,
			andiPar, R2, 0xFF ^ 1<<SCL, NAN, T1&&!T0,
			outiPar, OE_N, 0xFF ^ (0<<SDA | 1<<SCL), NAN, !T1&&!T0,
			nopPar,0,0,0,0
			);
	parallelEx(
			outPar, OE_N, R2, NAN, T0,
			outPar, OE_N, R2, NAN, T1&&!T0,
			wfePar, NAN, NAN, NAN, !T1&&!T0,
			nopPar,0,0,0,0
			);
	parallelEx(
			somPar, OE_N, SDA, R2, T0,
			somPar, OE_N, SDA, R2, T1&&!T0,
			outiPar, OE_N, 0xFF, NAN, !T1&&!T0,
			nopPar,0,0,0,0
			);
	parallelEx(
			wfePar, NAN, NAN, NAN, T0,
			wfePar, NAN, NAN, NAN, T1&&!T0,
			lslPar, RX_REG, 1, NAN, !T1&&!T0,
			nopPar,0,0,0,0
			);
	parallelEx(
			lslPar, TX_REG, 1, NAN, T0,
			lslPar, TX_REG, 1, NAN, T1&&!T0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			oriPar, R2, 1<<SCL, NAN, T0,
			oriPar, R2, 1<<SCL, NAN, T1&&!T0,
			silPar, SDA, R2, NAN, !T1&&!T0,
			nopPar,0,0,0,0
			);
	parallelEx(
			outPar, OE_N, R2, NAN, T0,
			outPar, OE_N, R2, NAN, T1&&!T0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
}

void i2cParallelAck(void)
{
	parallelEx(
			wfePar, NAN, NAN, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			andiPar, R2, 0xFF ^ 1<<SCL, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			outPar, OE_N, R2, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);

	parallelEx(
			outiPar, OE_N, 0xFF ^ (0<<SDA | 1<<SCL), NAN, T0, //release SDA to receive ack
			outiPar, OE_N, 0xFF ^ (0<<SDA | 1<<SCL), NAN, T1&&!T0, //release SDA to receive ack
			outiPar, OE_N, 0xFF ^ (1<<SDA | 1<<SCL), NAN, !T1&&!T0, //TODO: release SDA if stopping, else pull low for ack
			nopPar,0,0,0,0
			);
	parallelEx(
			wfePar, NAN, NAN, NAN, 1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			outiPar, OE_N, 0xFF ^ (0<<SDA | 0<<SCL), NAN, T0, //release SDA to receive ack
			outiPar, OE_N, 0xFF ^ (0<<SDA | 0<<SCL), NAN, T1&&!T0, //release SDA to receive ack
			outiPar, OE_N, 0xFF ^ (1<<SDA | 0<<SCL), NAN, !T1&&!T0, //TODO: release SDA if stopping, else pull low for ack
			nopPar,0,0,0,0
			);
	parallelEx(
			inPar, R1, NAN, NAN, T0,
			inPar, R1, NAN, NAN, T1&&!T0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			cpibe1Par, rT2, R1, SDA, T0, //not ack
			cpibe1Par, rT2, R1, SDA, T1&&!T0, //not ack
			clrPar,	   rT2, NAN,NAN, !T1&&!T0,
			nopPar,0,0,0,0
			);
	if(
	parallelEx2Jmp(
			jmp, i2cMNotAck, T2,
			jmp,NULL,0,//dummy
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			))
	{return;}

	//-----------------------------------------------------------
	//Simulator hack to fool system into being controlled by host
	i--;
	if(i != 0)
		{debugRegWriteBit(CTRL_REG, I2C_STOP, 1);}
	else
		{debugRegWriteBit(CTRL_REG, I2C_STOP, 0);}
	//-----------------------------------------------------------

	parallelEx(
			cpibe0Par, rT1, R2, 7, T0, //Read or write
			cpibe0Par, rT2, CTRL_REG, I2C_STOP, T1&&!T0, //Stop/rep
			cpibe0Par, rT2, CTRL_REG, I2C_STOP, !T1&&!T0, //Stop/rep
			nopPar,0,0,0,0
			);
	parallelEx(
			clrPar, rT0, NAN, NAN, T0, //turn off address
			//setPar, rT2, NAN, NAN, T1&&!T0, //Continue to stop/rep
			//setPar, rT2, NAN, NAN, !T1&&!T0, //Continue to stop/rep
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
}

void i2cMStop(void)
{
	parDummy();
	wfe();

	parDummy();
	outi(OE_N, 0xFF ^ 1<<SCL); //Pull SCL low
	//wait short
	parDummy();
	outi(OE_N, 0xFF ^ (1<<SDA | 1<<SCL)); //Pull SDA low

	parDummy();
	wfe();

	parDummy();
	outi(OE_N, 0xFF ^ 1<<SDA); //Release SCL

	parDummy();
	wfe();

	parDummy();
	outi(OE_N, 0xFF); //Release SDA

	parDummy();
	wfe();
}

void i2cMRepStart(void)
{
	parDummy();
	wfe();

	parDummy();
	outi(OE_N, 0xFF ^ 1<<SCL); //Pull SCL low
	//wait short
	parDummy();
	outi(OE_N, 0xFF ^ (0<<SDA | 1<<SCL)); //Release SDA

	parDummy();
	wfe();

	parDummy();
	outi(OE_N, 0xFF); //Release SCL

	parDummy();
	wfe();

	parDummy();
	outi(OE_N, 0xFF ^ 1<<SDA); //New Start

	parDummy();
	wfe();
}

void i2cParallelStopRep(void)
{
	//T3 == stop

	parDummy();
	wfe();

	parallelEx( //Cycle SCL low
			outiPar, OE_N, 0xFF ^ (0<<SDA | 1<<SCL), NAN, T3, //Ack was high when stopping
			outiPar, OE_N, 0xFF ^ (0<<SDA | 1<<SCL), NAN, !T3, //SDA was released when writing
			outiPar, OE_N, 0xFF ^ (1<<SDA | 1<<SCL), NAN, 0, //TODO: SDA was pulled low when reading
			nopPar,0,0,0,0

	);
	//wait short
	parallelEx(//Set SDA to prepare for stop/rep message
			outiPar, OE_N, 0xFF ^ (1<<SDA | 1<<SCL), NAN, T3, //SDA low
			outiPar, OE_N, 0xFF ^ (0<<SDA | 1<<SCL), NAN, !T3, //SDA high
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);

	parDummy();
	wfe();

	parallelEx(//Cycle SCL high
			outiPar, OE_N, 0xFF ^ (1<<SDA | 0<<SCL), NAN, T3, //SDA low
			outiPar, OE_N, 0xFF ^ (0<<SDA | 0<<SCL), NAN, !T3, //SDA high
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);

	parDummy();
	wfe();

	parallelEx(
			outiPar, OE_N, 0xFF, NAN, T3,			//Release SDA to stop
			outiPar, OE_N, 0xFF ^ 1<<SDA, NAN, !T3, //Pull SDA low to restart
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);

	parDummy();
	wfe();

	parallelEx2Jmp(
			jmp, i2cParallel, !T3,
			jmp,NULL,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);

}

void i2cMNotAck(void)
{
	fprintf(stderr, "i2cM:Message was not acknowledged!\n");
	//Some stop and exit feature
}
