/*
 * instr.c
 *
 *  Created on: Oct 8, 2015
 *      Author: heso
 */

#include "instr.h" //glob.h, condReg.h included
#include "ioMap.h"
#include "hex.h"
#include "regFile.h"
#include "vcd.h"
#include "report.h"
//#include "condReg.h"

//==============================================================
//IO
void in(unsigned int reg)
{
	reportInstr(I_IN);
	unsigned int val;
	val = ioRead();
	regWrite(reg, val);
	vcdWaitCycle();
}

void out(io_t padCtrl, unsigned int reg)
{
	reportInstr(I_OUT);
	unsigned int val;
	val = regRead(reg);
	ioWrite(padCtrl, val);
	vcdWaitCycle();
}

void outi(io_t padCtrl, unsigned int imm)
{
	reportInstr(I_OUTI);
	ioWrite(padCtrl, imm);
	vcdWaitCycle();
}

//==============================================================
//Logic
void andi(unsigned int regd, unsigned int imm)
{
	reportInstr(I_ANDI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald & imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void and(unsigned int regd, unsigned int rega)
{
	reportInstr(I_AND);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald & vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void ori(unsigned int regd, unsigned int imm)
{
	reportInstr(I_ORI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald | imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void or(unsigned int regd, unsigned int rega)
{
	reportInstr(I_OR);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald | vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void xor(unsigned int regd, unsigned int rega)
{
	reportInstr(I_XOR);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald ^ vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void xori(unsigned int regd, unsigned int imm)
{
	reportInstr(I_XORI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald ^ imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}

//==============================================================
//Shift
void lsr(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LSR);
	unsigned int vald;
	vald = regRead(regd);
	vald >>= imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void lsl(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LSL);
	unsigned int vald;
	vald = regRead(regd);
	vald <<= imm;
	vald = vald & 0xFF; //Mask 9th bit.
	regWrite(regd, vald);
	vcdWaitCycle();
}

//==============================================================
//Load
void ldi(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LDI);
	regWrite(regd, imm);
	vcdWaitCycle();
}

void mov(unsigned int regd, unsigned int rega)
{
	reportInstr(I_MOV);
	unsigned int val;
	val = regRead(rega);
	regWrite(regd, val);
	vcdWaitCycle();
}

//==============================================================
//SOL
void som(io_t padCtrl, unsigned int addr, unsigned int reg)
{
	reportInstr(I_SOM);
	unsigned int val, bus;
	val = regRead(reg);

	bus = regReadTxReg(MSB);
	if (bus)
	{
		bus >>= (7 - addr);
		val = val | bus;
	}
	else
	{
		val = val & ~(1 << addr);
	}
	ioWrite(padCtrl, val);
	regWrite(reg, val);
	vcdWaitCycle();
}

void sol(io_t padCtrl, unsigned int addr, unsigned int reg)
{
	reportInstr(I_SOL);
	unsigned int val, bus;
	val = regRead(reg);

	bus = regReadTxReg(LSB);

	if (bus)
	{
		bus <<= addr;
		val = val | bus;
	}
	else
	{
		val = val & ~(1 << addr);
	}
	ioWrite(padCtrl, val);
	regWrite(reg, val);
	vcdWaitCycle();
}

void sim(unsigned int addr, unsigned int reg)
{
	reportInstr(I_SIM);
	unsigned int bus;
	bus = ioRead();

	regWrite(reg, bus);

	bus <<= 7 - addr; //Shift addr-bit to MSB position
	bus &= 0x80; //Mask

	regWriteRxReg(MSB, bus);
	vcdWaitCycle();
}
void sil(unsigned int addr, unsigned int reg)
{
	reportInstr(I_SIL);
	unsigned int bus;
	bus = ioRead();

	regWrite(reg, bus);

	bus >>= addr; //Shift addr-bit to LSB position
	bus &= 0x01; //Mask

	regWriteRxReg(LSB, bus);
	vcdWaitCycle();
}

//==============================================================
//Skip
void cpse(unsigned int rega, unsigned int boolean)
{
	reportInstr(I_CPSE);
	vcdWaitCycle();
}
void cbse(unsigned int rega, unsigned int bit, unsigned int boolean)
{
	reportInstr(I_CBSE);
	vcdWaitCycle();
}

int cbseJmp(unsigned int rega, unsigned int bit, unsigned int boolean, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CBSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (((val & 1<<bit) && 1) != boolean)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();
	return 0;
}


void cpseLogic(unsigned int rega, unsigned int imm, void (*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2)
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();

	if (val != imm)
	{
		logicFunc(var1, var2);
	}
	else
		nop();
}

void cpsneLogic(unsigned int rega, unsigned int imm, void (*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2)
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();

	if (val == imm)
	{
		logicFunc(var1, var2);
	}
	else
		nop();
}

//Unnecessary to pass jmpFunc if there is only one type of jmp function.
int cpseJmp(unsigned int rega, unsigned int imm, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (val != imm)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();
	return 0;
}

int cpsneJmp(unsigned int rega, unsigned int imm, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (val == imm)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();
	return 0;
}
//==============================================================
//Misc control
void repeat(unsigned int iter, void (*section)())
{
	reportInstr(I_REPEAT);
	int i;
	vcdWaitCycle();
	for (i = 0; i < iter; i++)
	{
		section();
	}
}

void jmp(void (*section)())
{
	reportPipeFlush();
	reportInstr(I_JMP);
vcdPipeFlush();
	vcdWaitCycle();
	section();
}

void jmpDummy(char* destination)
{
	reportPipeFlush();
	reportInstr(I_JMP);
vcdPipeFlush();
	vcdWaitCycle();
}
//Set
void timst(unsigned int time)
{
	reportInstr(I_TIMST);
	vcdWaitCycle();
	vcdStartTimer(time);
}

void wfe(void)
{
	reportInstr(I_WFE);
	//Prefetch, put processor in sleep and wait.
	vcdWaitCycle();
	vcdWaitTimer();
}

//=============================================================
//NOP
void nop(void)
{
	reportInstr(I_NOP);
	vcdWaitCycle();
}


//===============================================================
//===========================PARALLEL============================
//===============================================================
int parallelEx(
		 void (*ex0)(unsigned int,unsigned int,unsigned int), unsigned int ex0var0, unsigned int ex0var1, unsigned int ex0var2, unsigned int ex0Cond
		,void (*ex1)(unsigned int,unsigned int,unsigned int), unsigned int ex1var0, unsigned int ex1var1, unsigned int ex1var2, unsigned int ex1Cond
		,void (*ex2)(unsigned int,unsigned int,unsigned int), unsigned int ex2var0, unsigned int ex2var1, unsigned int ex2var2, unsigned int ex2Cond
		,void (*ex3)(unsigned int,unsigned int,unsigned int), unsigned int ex3var0, unsigned int ex3var1, unsigned int ex3var2, unsigned int ex3Cond
){
	reportParallel();
	if(ex0Cond)
	{
		ex0(ex0var0, ex0var1, ex0var2);
	}
	else if(ex1Cond)
	{
		ex1(ex1var0, ex1var1, ex1var2);
	}
	else if(ex2Cond)
	{
		ex2(ex2var0, ex2var1, ex2var2);
	}
	else if(ex3Cond)
	{
		ex3(ex3var0, ex3var1, ex3var2);
	}
	else
	{
		nop();
	}
	return 0;
}

int parallelEx2Jmp(
		 //void (*ex0)(void (*jmpFunc)(void (*section)),void (*section)()), void (*jmpFunc)(void *section), void(*section), unsigned int ex0Cond
		void (*ex0)(void* (section)())  , void (*ex0section)()  , unsigned int ex0Cond
		,void (*ex1)(void* (section)())  , void (*ex1section)()  , unsigned int ex1Cond
		//void (*ex0)(void*), void(*ex0section), unsigned int ex0Cond
		//,void (*ex1)(void*), void(*ex1section), unsigned int ex1Cond
		,void (*ex2)(unsigned int,unsigned int,unsigned int), unsigned int ex2var0, unsigned int ex2var1, unsigned int ex2var2, unsigned int ex2Cond
		,void (*ex3)(unsigned int,unsigned int,unsigned int), unsigned int ex3var0, unsigned int ex3var1, unsigned int ex3var2, unsigned int ex3Cond
){
	reportParallel();
	if(ex0Cond)
	{
		ex0(ex0section);
		return 1;
	}
	else if(ex1Cond)
	{
		ex1(ex1section);
		return 1;
	}
	else if(ex2Cond)
	{
		ex2(ex2var0, ex2var1, ex2var2);
	}
	else if(ex3Cond)
	{
		ex3(ex3var0, ex3var1, ex3var2);
	}
	else
	{
		nop();
	}
	return 0;
}

int parallelExJmp(
		 //void (*ex0)(void (*jmpFunc)(void (*section)),void (*section)()), void (*jmpFunc)(void *section), void(*section), unsigned int ex0Cond
		void (*ex0)(void* (section)())  , void (*ex0section)()  , unsigned int ex0Cond
		,void (*ex1)(void* (section)())  , void (*ex1section)()  , unsigned int ex1Cond
		,void (*ex2)(void* (section)())  , void (*ex2section)()  , unsigned int ex2Cond
		,void (*ex3)(void* (section)())  , void (*ex3section)()  , unsigned int ex3Cond
		//void (*ex0)(void*), void(*ex0section), unsigned int ex0Cond
		//,void (*ex1)(void*), void(*ex1section), unsigned int ex1Cond
		//,void (*ex2)(unsigned int,unsigned int,unsigned int), unsigned int ex2var0, unsigned int ex2var1, unsigned int ex2var2, unsigned int ex2Cond
		//,void (*ex3)(unsigned int,unsigned int,unsigned int), unsigned int ex3var0, unsigned int ex3var1, unsigned int ex3var2, unsigned int ex3Cond
){
	reportParallel();
	if(ex0Cond)
	{
		ex0(ex0section);
		return 1;
	}
	else if(ex1Cond)
	{
		ex1(ex1section);
		return 1;
	}
	else if(ex2Cond)
	{
		ex2(ex2section);
		return 1;
	}
	else if(ex3Cond)
	{
		ex3(ex3section);
		return 1;
	}
	else
	{
		nop();
	}
	return 0;
}


void parDummy(void)
{
	reportParallel();
}

//=======================================================
//Conditional check
void cpiePar(unsigned int cReg, unsigned int reg, unsigned int imm)
{
	unsigned int val;
	val = regRead(reg);
	if(val == imm)
		condRegWrite(cReg, 1);
	else
		condRegWrite(cReg, 0);
	reportInstr(I_CPI);
	vcdWaitCycle();
}

void cpinePar(unsigned int cReg, unsigned int reg, unsigned int imm)
{
	unsigned int val;
	val = regRead(reg);
	if(val != imm)
		condRegWrite(cReg, 1);
	else
		condRegWrite(cReg, 0);
	reportInstr(I_CPI);
	vcdWaitCycle();
}

void setPar(unsigned int cReg, unsigned int dummy, unsigned int dummy1)
{
	condRegWrite(cReg, 1);
	reportInstr(I_SET);
	vcdWaitCycle();
}

void clrPar(unsigned int cReg, unsigned int dummy, unsigned int dummy1)
{
	condRegWrite(cReg, 0);
	reportInstr(I_CLR);
	vcdWaitCycle();
}

void cpibe1Par(condReg_t cReg, unsigned int reg, unsigned int bit)
{
	unsigned int val;
	val = regRead(reg);
	if((val & 1<<bit))
		condRegWrite(cReg, 1);
	else
		condRegWrite(cReg, 0);
	reportInstr(I_CPI);
	vcdWaitCycle();
}

void cpibe0Par(condReg_t cReg, unsigned int reg, unsigned int bit)
{
	unsigned int val;
	val = regRead(reg);
	if (!(val & 1 << bit))
		condRegWrite(cReg, 1);
	else
		condRegWrite(cReg, 0);
	reportInstr(I_CPI);
	vcdWaitCycle();
}
//=========================================================
//IO
void inPar(unsigned int reg, unsigned int dummy, unsigned int dummy1)
{
	reportInstr(I_IN);
	unsigned int val;
	val = ioRead();
	regWrite(reg, val);
	vcdWaitCycle();
}

void outPar(unsigned int padCtrl, unsigned int reg, unsigned int dummy)
{
	reportInstr(I_OUT);
	unsigned int val;
	val = regRead(reg);
	ioWrite(padCtrl, val);
	vcdWaitCycle();
}

void outiPar(unsigned int padCtrl, unsigned int imm, unsigned int dummy)
{
	reportInstr(I_OUTI);
	ioWrite(padCtrl, imm);
	vcdWaitCycle();
}

//==============================================================
//Logic
void andiPar(unsigned int regd, unsigned int imm, unsigned int dummy)
{
	reportInstr(I_ANDI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald & imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void andPar(unsigned int regd, unsigned int rega, unsigned int dummy)
{
	reportInstr(I_AND);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald & vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void oriPar(unsigned int regd, unsigned int imm, unsigned int dummy)
{
	reportInstr(I_ORI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald | imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void orPar(unsigned int regd, unsigned int rega, unsigned int dummy)
{
	reportInstr(I_OR);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald | vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void xorPar(unsigned int regd, unsigned int rega, unsigned int dummy)
{
	reportInstr(I_XOR);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald ^ vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void xoriPar(unsigned int regd, unsigned int imm, unsigned int dummy)
{
	reportInstr(I_XORI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald ^ imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}

//==============================================================
//Shift
void lsrPar(unsigned int regd, unsigned int imm, unsigned int dummy)
{
	reportInstr(I_LSR);
	unsigned int vald;
	vald = regRead(regd);
	vald >>= imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void lslPar(unsigned int regd, unsigned int imm, unsigned int dummy)
{
	reportInstr(I_LSL);
	unsigned int vald;
	vald = regRead(regd);
	vald <<= imm;
	vald = vald & 0xFF; //Mask 9th bit.
	regWrite(regd, vald);
	vcdWaitCycle();
}

//==============================================================
//Load
void ldiPar(unsigned int regd, unsigned int imm, unsigned int dummy)
{
	reportInstr(I_LDI);
	regWrite(regd, imm);
	vcdWaitCycle();
}

void movPar(unsigned int regd, unsigned int rega, unsigned int dummy)
{
	reportInstr(I_MOV);
	unsigned int val;
	val = regRead(rega);
	regWrite(regd, val);
	vcdWaitCycle();
}

//==============================================================
//SOL
void somPar(unsigned int padCtrl, unsigned int addr, unsigned int reg)
{
	reportInstr(I_SOM);
	unsigned int val, bus;
	val = regRead(reg);

	bus = regReadTxReg(MSB);
	if (bus)
	{
		bus >>= (7 - addr);
		val = val | bus;
	}
	else
	{
		val = val & ~(1 << addr);
	}
	ioWrite(padCtrl, val);
	regWrite(reg, val);
	vcdWaitCycle();
}

void solPar(unsigned int padCtrl, unsigned int addr, unsigned int reg)
{
	reportInstr(I_SOL);
	unsigned int val, bus;
	val = regRead(reg);

	bus = regReadTxReg(LSB);

	if (bus)
	{
		bus <<= addr;
		val = val | bus;
	}
	else
	{
		val = val & ~(1 << addr);
	}
	ioWrite(padCtrl, val);
	regWrite(reg, val);
	vcdWaitCycle();
}

void simPar(unsigned int addr, unsigned int reg, unsigned int dummy)
{
	reportInstr(I_SIM);
	unsigned int bus;
	bus = ioRead();

	regWrite(reg, bus);

	bus <<= 7 - addr; //Shift addr-bit to MSB position
	bus &= 0x80; //Mask

	regWriteRxReg(MSB, bus);
	vcdWaitCycle();
}
void silPar(unsigned int addr, unsigned int reg, unsigned int dummy)
{
	reportInstr(I_SIL);
	unsigned int bus;
	bus = ioRead();

	regWrite(reg, bus);

	bus >>= addr; //Shift addr-bit to LSB position
	bus &= 0x01; //Mask

	regWriteRxReg(LSB, bus);
	vcdWaitCycle();
}


void timstPar(unsigned int time, unsigned int dummy, unsigned int dummy1)
{
	reportInstr(I_TIMST);
	vcdWaitCycle();
	vcdStartTimer(time);
}

void wfePar(unsigned int dummy, unsigned int dummy1, unsigned int dummy2)
{
	reportInstr(I_WFE);
	//Prefetch, put processor in sleep and wait.
	vcdWaitCycle();
	vcdWaitTimer();
}


int jmpPar(void (*section)(), unsigned int dummy, unsigned int dummy1)
{
	reportPipeFlush();
	reportInstr(I_JMP);
vcdPipeFlush();
	vcdWaitCycle();
	section();
	return 1;
}

void jmpDummyPar(char* destination, unsigned int dummy, unsigned int dummy1)
{
	reportPipeFlush();
	reportInstr(I_JMP);
vcdPipeFlush();
	vcdWaitCycle();
}

//=============================================================
//NOP
void nopPar(unsigned int dummy1, unsigned int dummy2, unsigned int dummy3)
{
	reportInstr(I_NOP);
	vcdWaitCycle();
}


