/*
 * uart.c
 *
 *  Created on: Oct 15, 2015
 *      Author: heso
 */


#include "uart.h"
#include "instr.h" //Includes glob.h
#include "vcd.h"
#include "regFile.h"
#include "condReg.h"

/*-----------------------------------------------------------------------------------------
	UART with Repeat and SOL and conditionals
-----------------------------------------------------------------------------------------*/
void uartSetup(void)
{
	condRegReset();
	outi (IE, 1<<RXD);	//Enable input RX = in[0]
	outi (OE_N, 0<<TXD & 0xFF);	//Enable output TX = out[7]
	outi (OUT, 1<<TXD);	//Idle
	timst(UART_CK_TIME);		//Baud rate 250K (500kHz)
}

void uartFull(unsigned int cycles)
{
	int i;
	for (i=0; i<cycles; i++) //== jmp uartRun at the end of every run
	{
		uartRun();
	}
}

void uartRun(void)
{
	uartTx();
	uartRx();
	jmpDummy("uartRun");
}

void uartTx(void)
{
	//T3 == running TX
	//T2 == running txStart

	//T2&&!T3 == starting
	//!T2&&T3 == running
	//T2&&T3  == stopping
	//!T2&&!T3== idle

	wfe(); //Posedge

	//ex0 = check start
	//ex1 = while running
	//ex2 = stop

	parallelEx(
			cpiePar, rT2, CTRL_REG, 1<<TXSTART, !T3,
			cpiePar, rT2, R2, 0x00, T3,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			outiPar,OUT, 0<<TXD, NAN, T2&&!T3,
			solPar,OUT, TXD, R0, !T2&&T3,
			outiPar,OUT, 1<<TXD, NAN, T2&&T3,
			nopPar,0,0,0,0
			);
	parallelEx(
			ldiPar, R2, 0x80, NAN, T2&&!T3,
			lsrPar,TX_REG, 1, NAN, !T2&&T3,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			andiPar,CTRL_REG, 0xFF ^ 1<<TXSTART, NAN, T2&&!T3,
			lsrPar,R2, 1, NAN, !T2&&T3,
			nopPar,0,0,0,0,//oriPar, FLAG_REG, 1<<TX_DONE, NAN, T2&&T3,
			nopPar,0,0,0,0
			);
	parallelEx(
			setPar, rT3, NAN, NAN, T2&&!T3,
			clrPar, rT3, NAN, NAN, T2&&T3,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			clrPar, rT2, NAN, NAN, T2,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
}

void uartRx(void)
{
	//T1 == running RX
	//T0 == rxStart

	wfe(); //Read middle

	parallelEx(
			inPar,R1, NAN, NAN, !T1,
			lsrPar, RX_REG, 1, NAN, T1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			cpiePar, rT0, R1, 0<<RXD, !T1,
			simPar , RXD, R0, NAN, T1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			ldiPar, R3, 0x80, NAN, T0,
			lsrPar, R3, 1, NAN, T1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			setPar, rT1, NAN, NAN, T0,
			cpinePar,rT1, R3, 0x00, T1,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
	parallelEx(
			clrPar, rT0, NAN, NAN, T0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0,
			nopPar,0,0,0,0
			);
}
