/*
 * spi.c
 *
 *  Created on: Oct 15, 2015
 *      Author: heso
 */

#include "spi.h"
#include "instr.h" //Includes glob.h
#include "vcd.h"
#include "regFile.h"

//Note: Several different SPI-configurations exist. Rising/falling edge, leading/lagging edge, CS/CS_N etc.
//NOte: the host will initiate traffic by triggering some event in the event generator. In this code calling spiM is that event.
void spiM(int transfers)
{
	spiMStart();

	repeat(transfers, spiMTransferByte); //This will probably not work in actual implementation.

	spiMEnd();
}

void spiMInit(void)
{
	outi (OE_N, 0xFF ^ (1<<MOSI | 1<<SCKO | 1<<CSn));	//Enable outputs
	outi (IE, 1<<MISO);	//Enable input
	outi (OUT, 1<<CSn);
	timst(SPI_CK_TIME);
}

void spiMStart(void)
{
	ori(CTRL_REG, 1<<SPI_RUNNING);
	ldi (R0, 0<<SCKO | 0<<CSn);		//CS, !SCKO
	out (OUT, R0);
	wfe(); 				//Waiting to let slave initialize
}

void spiMTransferByte(void)
{
	andi(CTRL_REG, 0xFF ^ 1<<SPI_BYTE_READY);
	repeat(8, spiMRepeat);
	ori(CTRL_REG, 1<<SPI_BYTE_READY);
}

void spiMEnd(void)
{
//Stop bit
	andi (R0, 1<<MOSI | 0<<SCKO);    	//prev, !SCKO
	out (OUT, R0);
	wfe();
	ori(R0, 1<<CSn);
	out(OUT, R0);
	andi(CTRL_REG, 0xFF ^ 1<<SPI_RUNNING);
	wfe();				//CS low for at least half a period.
}

void spiMRepeat(void)
{
//Set MOSI
	mov(R0, TX_REG);
	andi(R0, 0x00 | 1<<MOSI |  0<<SCKO | 0<<CSn);
	out(OUT, R0);
	lsl (TX_REG, 1);	//Shift txreg left

//Cycle SCKO high
	wfe();
	ori (R0, 1<<SCKO); 		//CS, SCKO
	out (OUT, R0);
	lsl (RX_REG, 1);	//Shift rxReg left

//Read MISO
	in(R1);
	lsr(R1, MISO);  //Assuming MISO is the only activated input
	or(RX_REG, R1); //Asumming RX_REG = 0x00 at start.

	wfe();
}
