/*
 * uart.c
 *
 *  Created on: Oct 15, 2015
 *      Author: heso
 */


#include "uart.h"
#include "instr.h" //Includes glob.h

int i;

/*-----------------------------------------------------------------------------------------
	UART with Repeat and SOL
-----------------------------------------------------------------------------------------*/
void uartSetup(void)
{
	outi (IE, 1<<RXD);	//Enable input RX = in[0]
	outi (OE_N, 1<<TXD ^ 0xFF);	//Enable output TX = out[7]
	outi (OUT, 1<<TXD);	//Idle
	timst(UART_CK_TIME);
}

//========================================================================================
//CTRL_REG [TXRUN, RXRUN, TXSTART]
void uartFull(int cycles)
{
	//uart:
	for (i = 0; i < cycles; i++)
	{
		uartTxIdle();
	}
}


void uartTxIdle(void)
{
	wfe();
	cbseJmp(CTRL_REG, TXSTART, 0, jmp, uartTxStart);

	wfe(); //Middle
	if(cbseJmp(CTRL_REG, RXRUN, 0, jmp, uartRxRun))
	{
		return;
	}


	in(R3);
	if( cbseJmp(R3, RXD, 1, jmp, uartRxStart) )
	{
		return;
	}
	uartIdleEnd();
}

void uartIdleEnd(void)
{
	if (cbseJmp(CTRL_REG, TXRUN, 0, jmp, uartTx))
		return;
	jmpDummy("uartIdle");
}

void uartTxStart(void)
{
	outi(OUT, 0<<TXD);
	ori(CTRL_REG, 1<<TXRUN);
	andi(CTRL_REG, 0xFF ^ 1<<TXSTART); //Clear start

//Continue to uartRx
	jmpDummy("uartRx");
}

void uartTx(void)
{
	repeat(8, uartTxRun);
	i += 8; //DEBUG control, add 8 cycles..
	uartTxStop();
}

void uartTxRun(void)
{
	wfe();
//TX
	mov(R0, TX_REG);
	lsl(R0, TXD); //TXD is the only output rest is don't care.
	out(OUT, R0);

	lsr (TX_REG, 1);
	lsr (R1, 1); //If RXrunning, shift. Else don't care.
	wfe();
	if(cbseJmp(CTRL_REG, RXRUN, 0, jmp, uartRxRunRep)) //Length 6+flush
	{
		return;
	}

	in(R3);
	if( cbseJmp(R3, RXD, 1, jmp, uartRxStartRep) ) //Length 3+flush
	{
		return;
	}
	nop(); //Idle
	nop(); //Idle
	nop(); //Idle
	nop(); //Idle
	nop(); //Idle
}

void uartTxStop(void)
{
	outi(OUT, 1<<TXD);
	andi(CTRL_REG, 0xFF ^ 1<<TXRUN); //Clear run
	jmpDummy("uartRx");
}

void uartRxStartRep(void)
{
	ldi(R1, 0x80);
	ori(CTRL_REG, 1<<RXRUN);
	nop();
}

void uartRxRunRep(void)
{
//Read RXD
	lsr (RX_REG, 1);
	in(R3);
	lsl(R3, 7-RXD); //RXD is the only input
	or(RX_REG, R3); //Assuming RX_REG=0 at start of every transfer

//Stop
	cpsneLogic(R1, 0x00, andi, CTRL_REG, 0xFF ^ 1<<RXRUN); //Clearing RXRUN indicates that data is ready
}

void uartRxRun(void)
{
	//Read RXD
	lsr (RX_REG, 1);
	lsr (R1, 1);
	in(R3);
	lsl(R3, 7-RXD); //RXD is the only input
	or(RX_REG, R3); //Assuming RX_REG=0 at start of every transfer

//Stop
	cpsneLogic(R1, 0x00, andi, CTRL_REG, 0xFF ^ 1<<RXRUN); //Clearing RXRUN indicates that data is ready
	jmp(uartIdleEnd);
}

void uartRxStart(void)
{
	ldi(R1, 0x80);
	ori(CTRL_REG, 1<<RXRUN);
	jmp(uartIdleEnd);
}
