/*
 * spi.c
 *
 *  Created on: Oct 15, 2015
 *      Author: heso
 */

#include "spi.h"
#include "instr.h" //Includes glob.h
#include "vcd.h"
#include "regFile.h"

//Note: Several different SPI-configurations exist. Rising/falling edge, leading/lagging edge, CS/CS_N etc.
//NOte: the host will initiate traffic by triggering some event in the event generator. In this code calling spiM is that event.
void spiM(int transfers)
{
	spiMStart();

	int i;
	for(i=0; i < transfers; i++)
	{
		spiMTransferByte();
	}
	spiMEnd();
}

void spiMInit(void)
{
	outi (OE_N, 0xFF ^ (1<<MOSI | 1<<SCKO | 1<<CSn));	//Enable outputs
	outi (IE, 1<<MISO);	//Enable input
	timst(SPI_CK_TIME);
}

void spiMStart(void)
{
	ori(CTRL_REG, 1<<SPI_RUNNING);
	ldi (R0, 0x00);		//CS, !SCKO
	out (OUT, R0);
	wfe(); 				//Waiting to let slave initialize
}

void spiMTransferByte(void)
{
	andi(CTRL_REG, 0xFF ^ 1<<SPI_BYTE_READY);

	ldi(RX_REG, 0x00); //Reset RX_REG
	ldi(R2, 0x80);
	spiMRepeat();

	ori(CTRL_REG, 1<<SPI_BYTE_READY);
}

void spiMEnd(void)
{
//Stop bit
	out (OUT, R0);
	wfe();
	ori(R0, 1<<CSn);//!CSn
	out (OUT, R0);
	andi(CTRL_REG, 0xFF ^ 1<<SPI_RUNNING);
	wfe();				//CS low for at least half a period.
}

void spiMRepeat(void)
{
//Set MOSI
	som (OUT, MOSI, R0);	//port 0 = {MSB, t0[6:0]}
	lsl (TX_REG, 1);	//Shift txreg left
	wfe();
	lsr (R2, 1); //Shift counter right

//Cycle SCKO high
	ori (R0, 1<<SCKO); 		//CS, SCKO
	out (OUT, R0);

//Read MISO
	lsl (RX_REG, 1);	//Shift rxReg left
	sil (MISO, R1);

//Cycle SCKO low
	wfe();
	andi (R0, 0<<SCKO | 1<<MOSI | 0<<CSn);

	cpseJmp(R2, 0x00, jmp, spiMRepeat);
}
