/*
 * uart.c
 *
 *  Created on: Oct 15, 2015
 *      Author: heso
 */


#include "uart.h"
#include "instr.h" //Includes glob.h

int i;

/*-----------------------------------------------------------------------------------------
	UART with Repeat and SOL
-----------------------------------------------------------------------------------------*/
void uartSetup(void)
{
	outi (IE, 1<<RXD);	//Enable input RX = in[0]
	outi (OE_N, 1<<TXD ^ 0xFF);	//Enable output TX = out[7]
	outi (OUT, 1<<TXD);	//Idle
	timst(UART_CK_TIME);		//Baud rate
}

//========================================================================================
//CTRL_REG [TXRUN, RXRUN, TXSTART]
void uartFull(int cycles)
{
	for (i = 0; i < cycles; i++)
	{
		uartTxIdle();
	}
}


void uartTxIdle(void)
{
	wfe();
	cbseJmp(CTRL_REG, TXSTART, 0, jmp, uartTxStart);

	wfe(); //Middle
	if(cbseJmp(CTRL_REG, RXRUN, 0, jmp, uartRxRun))
	{
		return;
	}


	in(R3);
	if( cbseJmp(R3, RXD, 1, jmp, uartRxStart) )
	{
		return;
	}
	uartIdleEnd();
}

void uartIdleEnd(void)
{
	if (cbseJmp(CTRL_REG, TXRUN, 0, jmp, uartTx))
		return;
	jmpDummy("uartIdle");
}

void uartTxStart(void)
{
	outi(OUT, 0<<TXD);
	ori(CTRL_REG, 1<<TXRUN);
	andi(CTRL_REG, 0xFF ^ 1<<TXSTART); //Clear start
//Continue to uartRx
	jmpDummy("uartRx");
}

void uartTx(void)
{
	repeat(8, uartTxRun);
	i += 8; //DEBUG control, add 8 cycles..
	uartTxStop();
}

void uartTxRun(void)
{
	wfe();
	sol (OUT, TXD, R0);
	lsr (TX_REG, 1);
	lsr (R1, 1); //If running RX, shift. Else: don't care.

	wfe();
	if(cbseJmp(CTRL_REG, RXRUN, 0, jmp, uartRxRunRep)) //Length 4 + flush
	{
		return;
	}

	in(R3);	 					   	//1
	if( cbseJmp(R3, RXD, 0, 		//2
			jmp, uartNop) ) 		//3 //Length 1 + flush
	{
		return;
	}
	ldi(R1, 0x80);					//4
	ori(CTRL_REG, 1<<RXRUN);		//5
	nop(); 							//6
}

void uartTxStop(void)
{
	outi(OUT, 1<<TXD);
	andi(CTRL_REG, 0xFF ^ 1<<TXRUN); //Clear run
	jmpDummy("uartRx");
}

void uartNop(void)
{
	nop(); //6
}

void uartRxRunRep(void)
{
//Read RXD
	lsr (RX_REG, 1); 					//3
	sim (RXD, R3);						//4
//Stop
	cpsneLogic(R1, 0x00, andi,  		//5
			CTRL_REG, 0xFF ^ 1<<RXRUN); //6  //Clearing RXRUN indicates that data is ready
}

void uartRxRun(void)
{
//Read RXD
	lsr (RX_REG, 1);
	lsr (R1, 1);
	sim (RXD, R3);
//Stop
	cpsneLogic(R1, 0x00, andi, CTRL_REG, 0xFF ^ 1<<RXRUN); //Clearing RXRUN indicates that data is ready
	jmp(uartIdleEnd);
}

void uartRxStart(void)
{
	ldi(R1, 0x80);
	ori(CTRL_REG, 1<<RXRUN);
	jmp(uartIdleEnd);
}
