/*
 * instr.c
 *
 *  Created on: Oct 8, 2015
 *      Author: heso
 */

#include "instr.h" //glob.h included
#include "ioMap.h"
#include "hex.h"
#include "regFile.h"
#include "vcd.h"
#include "report.h"

//==============================================================
//IO
void in(unsigned int reg)
{
	reportInstr(I_IN);
	unsigned int val;
	val = ioRead();
	regWrite(reg, val);
	vcdWaitCycle();
}

void out(io_t padCtrl, unsigned int reg)
{
	reportInstr(I_OUT);
	unsigned int val;
	val = regRead(reg);
	ioWrite(padCtrl, val);
	vcdWaitCycle();
}

void outi(io_t padCtrl, unsigned int imm)
{
	reportInstr(I_OUTI);
	ioWrite(padCtrl, imm);
	vcdWaitCycle();
}

//==============================================================
//Logic
void andi(unsigned int regd, unsigned int imm)
{
	reportInstr(I_ANDI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald & imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void and(unsigned int regd, unsigned int rega)
{
	reportInstr(I_AND);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald & vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void ori(unsigned int regd, unsigned int imm)
{
	reportInstr(I_ORI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald | imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void or(unsigned int regd, unsigned int rega)
{
	reportInstr(I_OR);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald | vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void xor(unsigned int regd, unsigned int rega)
{
	reportInstr(I_XOR);
	unsigned int vald, vala;
	vald = regRead(regd);
	vala = regRead(rega);
	vald = vald ^ vala;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void xori(unsigned int regd, unsigned int imm)
{
	reportInstr(I_XORI);
	unsigned int vald;
	vald = regRead(regd);
	vald = vald ^ imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}

//==============================================================
//Shift
void lsr(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LSR);
	unsigned int vald;
	vald = regRead(regd);
	vald >>= imm;
	regWrite(regd, vald);
	vcdWaitCycle();
}
void lsl(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LSL);
	unsigned int vald;
	vald = regRead(regd);
	vald <<= imm;
	vald = vald & 0xFF; //Mask 9th bit.
	regWrite(regd, vald);
	vcdWaitCycle();
}

//==============================================================
//Load
void ldi(unsigned int regd, unsigned int imm)
{
	reportInstr(I_LDI);
	regWrite(regd, imm);
	vcdWaitCycle();
}

void mov(unsigned int regd, unsigned int rega)
{
	reportInstr(I_MOV);
	unsigned int val;
	val = regRead(rega);
	regWrite(regd, val);
	vcdWaitCycle();
}

//==============================================================
//Skip
void cpse(unsigned int rega, unsigned int boolean)
{
	reportInstr(I_CPSE);
	vcdWaitCycle();
}
void cbse(unsigned int rega, unsigned int bit, unsigned int boolean)
{
	reportInstr(I_CBSE);
	vcdWaitCycle();
}

int cbseJmp(unsigned int rega, unsigned int bit, unsigned int boolean, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CBSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (((val & 1<<bit) && 1) != boolean)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();
	return 0;
}

void cbseLogic(unsigned int rega, unsigned int bit, unsigned int boolean, void(*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2)
{
	reportInstr(I_CBSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();

	if  (((val & 1<<bit) && 1) != boolean)
	{
		logicFunc(var1, var2);
	}
	else
		nop();

}

void cpseLogic(unsigned int rega, unsigned int imm, void (*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2)
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();

	if (val != imm)
	{
		logicFunc(var1, var2);
	}
	else
		nop();

}

void cpsneLogic(unsigned int rega, unsigned int imm, void (*logicFunc)(unsigned int, unsigned int), unsigned int var1, unsigned int var2)
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();

	if (val == imm)
	{
		logicFunc(var1, var2);
	}
	else
		nop();

}

int cpseJmp(unsigned int rega, unsigned int imm, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (val != imm)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();

	return 0;
}

int cpsneJmp(unsigned int rega, unsigned int imm, void (*jmpFunc)(), void (*section)())
{
	reportInstr(I_CPSE);
	unsigned int val;
	val = regRead(rega);
	vcdWaitCycle();
	if (val == imm)
	{
		jmpFunc(section);
		return 1;
	}
	else
		nop();

	return 0;
}

void jmp(void (*section)())
{
	reportPipeFlush();
	reportInstr(I_JMP);
vcdPipeFlush();
	vcdWaitCycle();
	section();
}

void jmpDummy(char* destination)
{
	reportPipeFlush();
	reportInstr(I_JMP);
vcdPipeFlush();
	vcdWaitCycle();
}
//Set
void timst(unsigned int time)
{
	reportInstr(I_TIMST);
	vcdWaitCycle();
	vcdStartTimer(time);
}

void wfe(void)
{
	reportInstr(I_WFE);
	//Prefetch, put processor in sleep and wait.
	vcdWaitCycle();
	vcdWaitTimer();
}

//==============================================================

void nop(void)
{
	reportInstr(I_NOP);
	vcdWaitCycle();
}
