/*
 * i2c.c
 *
 *  Created on: Nov 10, 2015
 *      Author: heso
 */

#include "i2c.h"
#include "instr.h" //Includes glob.h
#include <stdio.h>


int i;

void i2cMInit(void)
{
	outi(OUT, 0x00); //Set normal output to zero, i2c will switch between connecting and disconnecting
	outi(OE_N, 0xFF); //Disconnect from bus, default.
	ldi(R0, 0x7F); //Initialize out register
	outi(IE, 1<<SCL | 1<<SDA); //Enable both as inputs. (inouts)
	timst(I2C_CK_TIME);
}
void i2cM(int transfers)
{
	i = transfers;
	//Start condition
	outi(OE_N, 0xFF ^ 1<<SDA);
	wfe();
	outi(OE_N, 0xFF ^ (1<<SDA | 1<<SCL));

	i2cMSendAddr();
}

void i2cMSendAddr(void)
{
	//repeat 8
	ldi(R3, 0x80);
	i2cMSendBitAddr();
	//Cycle ACK
	wfe();
	andi(R0, R0, 0xFF ^ 1<<SCL);
	out(OE_N, R0); //Pull SCL low
	//wait short
	outi(OE_N, 0xFF ^ (0<<SDA | 1<<SCL)); //Release SDA
	wfe();
	outi(OE_N, 0xFF); //Release SCL
	i2cMAddrAck();
}

void i2cMSendBitAddr(void)
{
//Clock stretch
//	in(R1);
//	if( cbseJmp(R1, SCL, 1, jmp, i2cMSendBitAddr)) //Wait for SCL to be released
//			return;

//Cycle SCL
	wfe();
	andi(R0, R0, 0xFF ^ 1<<SCL); //Pull SCL low while holding SDA equal.
	out(OE_N, R0); //Set OE_N to pull down SCL

//Set SDA
	andi(R0, TX_REG, 0xFF ^ 1<<SCL); //SDA = TX[7], SCL = 0, rest is don't care
	ori(R0, R0, 0xFF ^ (1<<SDA | 1<<SCL)); //May be removed if lower bits is in fact don't care
	out(OE_N, R0);
	lsr(R3, R3, 1); //count reg
	lsl(TX_REG, TX_REG, 1);

//Cycle SCL
	wfe();
	ori(R0, R0, 1<<SCL); //Release SCL
	out(OE_N, R0);

//Check that master wins arbitration
//	in(R1);
//	xor(R1, R0);
//	if( cpseJmp(R1, 0x00, jmp, i2cMArbLost))
//		return;

//Loop
	cpseJmp(R3, 0x00, jmp, i2cMSendBitAddr);
}

void i2cMAddrAck(void)
{
//Clock Stretching
	in(R1);
//	if( cbseJmp(R1, SCL, 1, jmp, i2cMAddrAck)) //Wait for SCL to be released
//		return;

//Receive ack
	if( cbseJmp(R1, SDA, 0, jmp, i2cMNotAck)) //If not ack
		return;

	if( cbseJmp(R0, SDA, 1, jmp, i2cMSendByte)) 	//R0 has stored the final bit
		return;
	i2cMReceiveByte();//continue to i2cMReceiveByte
}

void i2cMReceiveByte(void)
{
	ldi(R3, 0x80);     //Init counter
	ldi(RX_REG, 0x00); //Init RX_REG
	i2cMReceiveBit();
	i2cMRxAck();
}

void i2cMReceiveBit(void)
{
//Clock Stretch
//	in(R1);
//	if( cbseJmp(R1, SCL, 1, jmp, i2cMReceiveBit)) //Wait for SCL to be released
//		return;

	wfe();

//Cycle SCL
	outi(OE_N, 0xFF ^ 1<<SCL); //Pull SCL low while not connecting to SDA
	lsl(RX_REG, RX_REG, 1);
	lsr(R3, R3, 1); //count reg
	wfe();
	outi(OE_N, 0xFF); //Release SCL (and SDA)

//Read
	in(R0);
	lsr(R0, R0, SDA);
	or(RX_REG, RX_REG, R0); //Assuming RX_REG = 0x00 at start.

//Loop control
	cpseJmp(R3, 0x00, jmp, i2cMReceiveBit);
}

void i2cMRxAck(void)
{
//Clock Stretching
//	in(R1);
//	if( cbseJmp(R1, SCL, 1, jmp, i2cMRxAck)) //Wait for SCL to be released
//		return;

//Send ack
	wfe();
	andi(R0, R0, 0xFF ^ 1<<SCL);
	out(OE_N, R0); //Pull SCL low
	outi(OE_N, 0xFF ^ (0<<SDA | 1<<SCL)); //Set SDA high

//Cycle SCL
	wfe();
	outi(OE_N, 0xFF ^ (0<<SDA | 0<<SCL)); //Release SCL

	/*  Simulator workaround */
	i--;
	cbse(CTRL_REG, I2C_STOP, 0); //Dummy for checking
	if(i == 0)
	{
		jmp(i2cMStop);
		return;
	}
	else
	{
		jmp(i2cMReceiveByte);
		return;
	}
//	Actual assembly code for workaround
//	if( cbseJmp(CTRL_REG, I2C_STOP, 0, jmp, i2cMReceiveByte)) //Some status bit indicating that more bytes are incoming.
//		return;
//	jmp(i2cMStop);
}

void i2cMSendByte(void)
{
	ldi(R3, 0x80); //Init counter
	i2cMSendBit();

	//Check ACK
	i2cMTxAck();
	//i2cMStop();
}

void i2cMSendBit(void)
{
//	in(R1);
//	if( cbseJmp(R1, SCL, 1, jmp, i2cMSendBit)) //Wait for SCL to be released
//		return;
	//Cycle SCL
	wfe();

	andi(R0, R0, 0xFF ^ 1<<SCL); //Pull SCL low while holding SDA equal.
	out(OE_N, R0); //Set OE_N
	//wait short

	//Set SDA
	//som(OE_N, SDA, R0);
	andi(R0, TX_REG, 0xFF ^ 1<<SCL); //SDA = TX[7], SCL = 0, rest is don't care
	ori(R0, R0, 0xFF ^ (1<<SDA | 1<<SCL)); //May be removed if lower bits is in fact don't care
	out(OE_N, R0);

	lsl(TX_REG, TX_REG, 1);
	lsr(R3, R3, 1); //count reg

	wfe();

	ori(R0, R0, 1<<SCL); //Release SCL
	out(OE_N, R0);

	//Check that master wins arbitration
//	in(R1);
//	xor(R1, R0);
//	if( cpseJmp(R1, 0x00, jmp, i2cMArbLost))
//		return;

	//Loop
	cpseJmp(R3, 0x00, jmp, i2cMSendBit);
}

void i2cMTxAck(void)
{
	//Cycle ACK
	wfe();
	andi(R0, R0, 0xFF ^ 1<<SCL);
	out(OE_N, R0); //Pull SCL low
	//wait short
	outi(OE_N, 0xFF ^ (0<<SDA | 1<<SCL)); //Release SDA
	wfe();
	outi(OE_N, 0xFF); //Release SCL and

//Clock Stretching
	in(R1);
//	if( cbseJmp(R1, SCL, 1, jmp, i2cMTxAck)) //Wait for SCL to be released
//		return;

//Receive ack
	if( cbseJmp(R1, SDA, 0, jmp, i2cMNotAck)) 	//If not ack
		return;

	/* Simulator workaround */
	i--;
	cbse(CTRL_REG, I2C_STOP, 0); //Dummy for checking
	if(i == 0)
	{
		jmp(i2cMStop);
		return;
	}
	else
	{
		jmp(i2cMSendByte);
		return;
	}

//	Actual assembly code for workaround
//	if( cbseJmp(CTRL_REG, I2C_STOP, 0, jmp, i2cMSendByte)) //Some status bit indicating that more bytes are incoming.
//		return;
//	jmp(i2cMStop);
}

void i2cMStop(void)
{
	wfe();
	outi(OE_N, 0xFF ^ 1<<SCL); //Pull SCL low
	outi(OE_N, 0xFF ^ (1<<SDA | 1<<SCL)); //Pull SDA low
	wfe();
	outi(OE_N, 0xFF ^ 1<<SDA); //Release SCL
	wfe();
	outi(OE_N, 0xFF); //Release SDA
	wfe();
}

void i2cMArbLost(void)
{
	fprintf(stderr, "i2cM:Master lost arbitration!\n");
	//Some stop and exit feature
}

void i2cMNotAck(void)
{
	fprintf(stderr, "i2cM:Message was not acknowledged!\n");
	//Some stop and exit feature
}
