/*
 * uart.c
 *
 *  Created on: Oct 15, 2015
 *      Author: heso
 */

#include "uart.h"
#include "instr.h" //Includes glob.h
#include "vcd.h"
#include "regFile.h"

void uartSetup(void)
{
	outi (IE, 1<<RXD);	//Enable input RX = in[0]
	outi (OE_N, 1<<TXD ^ 0xFF);	//Enable output TX = out[7]
	outi (OUT, 1<<TXD);	//Idle
	timst(UART_CK_TIME);		//Baud rate 250K (500kHz)
}

//========================================================================================
//CTRL_REG [TXRUN, RXRUN, TXSTART]
/*
 * R0 == temp in/out reg
 * R1 == rx counter
 * R2 == tx counter
 * R3 == unused
 */

void uartFull(int cycles)
{
	int i;
	for (i = 0; i < cycles; i++)
	{
		uartTx();
		uartRx();
	}
}

void uartTx(void)
{
	wfe();//Posedge
	if(cbseJmp(CTRL_REG, TXRUN, 0, jmp, uartTxRun))
	{
		return;
	}
	if (cbseJmp(CTRL_REG, TXSTART, 0, jmp, uartTxStart))
	{
		return;
	}
}

void uartRx(void)
{
	wfe(); //Middle
	if(cbseJmp(CTRL_REG, RXRUN, 0, jmp, uartRxRun))
	{
		return;
	}

	in(R0);
	if( cbseJmp(R0, RXD, 1, jmp, uartRxStart) )
	{
		return;
	}
	jmpDummy("uart");
}

void uartTxStart(void)
{
	outi(OUT, 0<<TXD);
	ori(CTRL_REG, CTRL_REG, 1<<TXRUN);
	andi(CTRL_REG, CTRL_REG, 0xFF ^ 1<<TXSTART); //Clear start
	ldi (R2, 0x80);
	//Continue to uartRx
	jmpDummy("uartRx");
}

void uartTxRun(void)
{
	//Check stop
	if( cpsneJmp(R2, 0x00, jmp, uartTxStop) )
	{
		return;
	}

	lsl(R0, TX_REG, TXD); //TXD is the only output rest is don't care.
	out(OUT, R0);

	lsr (TX_REG, TX_REG, 1);
	lsr (R2, R2, 1);
	jmpDummy("uartRx");
}

void uartTxStop(void)
{
	outi(OUT, 1<<TXD);
	andi(CTRL_REG, CTRL_REG, 0xFF ^ 1<<TXRUN); //Clear run
	jmpDummy("uartRx");
}

void uartRxRun(void)
{
	//Read RXD
	lsr (RX_REG, RX_REG, 1);
	lsr (R1, R1, 1);
	//sim (RXD, R0);
	in(R0);
	lsl(R0, R0, 7-RXD); //RXD is the only input
	or(RX_REG, RX_REG, R0); //Assuming RX_REG=0 at start of every transfer

//Stop
	cpsne3Op(R1, 0x00, andi, CTRL_REG, CTRL_REG, 0xFF ^ 1<<RXRUN); //Clearing RXRUN indicates that data is ready
	jmpDummy("uart");
}

void uartRxStart(void)
{
	ldi(R1, 0x80);
	ldi(RX_REG, 0x00);
	ori(CTRL_REG, CTRL_REG, 1<<RXRUN);
	jmpDummy("uart");
}
