/*
 * iomap.c
 *
 *  Created on: Oct 8, 2015
 *      Author: heso
 */

#include "ioMap.h"
#include "vcd.h"
#include "glob.h"
#include <stdio.h>
#include <string.h>
#include <limits.h>

static unsigned int ioArray[4][IO_SIZE];
static unsigned int ioIter[4];
unsigned int oenReg;
unsigned int ieReg;

void ioWrite (io_t padCtrl, unsigned int imm)
{
	vcdIo(padCtrl, imm);
	switch(padCtrl){
	case(OE_N):
		oenReg = imm;
		ioArray[padCtrl][ioIter[padCtrl]++] = imm;
		break;
	case(IE):
		ieReg = imm;
		ioArray[padCtrl][ioIter[padCtrl]++] = imm;
		break;
	case(OUT):
		ioArray[padCtrl][ioIter[padCtrl]++] = imm & ~oenReg;
		break;
	default:
		fprintf(stderr, "ioWrite: Tried to write to non-writable padCtrl");
		break;
	}
}

unsigned int ioRead	(void)
{
	unsigned int val;
	val = ieReg & ioArray[IN][ioIter[IN]++];
	vcdIo(IN, val);
	return val;
}

void initInArray(void)
{
	int i;
	for (i = 0xFF; i>=0; i--)
		ioArray[IN][i] = i;
}

void ioMapReset(void)
{
	memset(ioArray, 0, sizeof(ioArray[0][0])*4*IO_SIZE);
	memset(ioIter, 0, sizeof(ioIter[0])*4);
	oenReg = 0xFF;
	ieReg = 0x00;
}

/* Can be used to fill ioArray[IN] with ones for I2C*/
void ioMapInitAllVal(unsigned int padCtrl, unsigned int val)
{
	int i;
	for(i = 0; i < IO_SIZE; i++)
	{
		ioArray[padCtrl][i] = val;

	}
}

void setInArray(unsigned int value, sb_t sbFirst, unsigned int dataw, unsigned int length, unsigned int offset)
{
	unsigned int bit = 0;
	unsigned int i = 0;
	for (i = 0; i<length;i++)
	{
		if(sbFirst == LSB)
		{
			bit = value & 1<<i;
		}
		else if(sbFirst == MSB)
		{
			bit = value & 1<<(length-1-i);
		}

		if(bit)
		{
			ioArray[IN][offset+i] |=  1<<dataw;
		}
		else
		{
			ioArray[IN][offset+i] &=  UINT_MAX ^ 1<<dataw;
		}
	}
}

unsigned int ioMap_GetDataPosedge(io_t padCtrl, sb_t sb, unsigned int dataw, unsigned int ckw, unsigned int length, unsigned int offset)
{
	unsigned int data = 0;
	unsigned int i = 0, j = 0;
	while (i < length)
	{
		if (((ioArray[padCtrl][offset + j] & 1 << ckw))
				&& !(ioArray[padCtrl][offset + j - 1] & 1 << ckw)) //@posedge(clk)
		{
			if (ioArray[padCtrl][offset + j] & 1 << dataw)
			{
				if (sb == MSB)
					data += 1 << (length - 1 - i);

				else
					data += 1 << i;
			}
			i++;
		}
		j++;
		if (j >= IO_SIZE)
		{
			fprintf(stderr,
					"ioMap_GetDataPosedge is trying to access data outside IO_SIZE\n");
			return 0xDEADBEEF;
		}
	}
	return data;
}

unsigned int ioMap_GetData(io_t padCtrl, sb_t sb, unsigned int dataw, unsigned int length, unsigned int offset)
{
	unsigned int data = 0;
	unsigned int i = 0;

	for (i = 0; i < length; i++)
	{
		if (ioArray[padCtrl][offset + i] & 1 << dataw)
		{
			if (sb == MSB)
				data += 1 << (length - 1 - i);
			else
				data += 1 << i;

		}
	}
	return data;
}
