/*
 * report.c
 *
 *  Created on: Oct 20, 2015
 *      Author: heso
 */

#include "report.h" //glob.h included
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>


const char *instrNameArray[] =
{ 	"in",
	"out",
	"outi",
	"andi",
	"and",
	"ori",
	"or",
	"xor",
	"xori",
	"lsr",
	"lsl",
	"ld",
	"ldi",
	"som",
	"sol",
	"sim",
	"sil",
	"cpse",
	"cbse",
	"repeat",
	"jmp",
	"timst",
	"wfe",
	"clrf",
	"setf",
	"mov",
	"cpi",
	"cpib",
	"nop",
	"set",
	"clr"
};

static int instrCount[I_SIZEOF];
static int cycleTime;
static int sleepTime;
static int pipeFlush;
static int parallelEx;
static FILE *repf;

void reportInit(char const* testName)
{
	//Print the time to differentiate different runs.
	time_t current_time;
	struct tm * time_info;
	char timeString[10]; // space for "DDD:HH:MM\0"

	time(&current_time);
	time_info = localtime(&current_time);

	strftime(timeString, sizeof(timeString), "%j-%H-%M", time_info);

	//-------------------------------------------------------------------------------
	//Create  filename

	//With time indicated in filename for multiple runs
	char* folder = "repFiles";
	char* name = "instrSimReport";
	char* extension = ".csv";


//	Time in filename removed
//	char fileSpec[strlen(name) + strlen(testName) + strlen(timeString) + strlen(extension) + 1];
//	snprintf(fileSpec, sizeof(fileSpec), "%s%s%s", name, testName, timeString, extension);

	char fileSpec[strlen(folder) + 1 + strlen(name) + strlen(testName) + strlen(extension) + 1];
	snprintf(fileSpec, sizeof(fileSpec), "%s/%s%s%s", folder, name, testName, extension);

	mkdir(folder, S_IRWXU | S_IRWXG | S_IRWXO);

	repf = fopen(fileSpec, "w");
	if (repf == NULL)
		printf("Error opening file.\n");
	else
		printf("filename: %s\n", fileSpec);
	//-------------------------------------------------------------------------------
	//No comments in csv-files
	fputs(
			"Group,Name,Value\n"
			,repf);
}

void reportInstr(instr_t instr)
{
	instrCount[instr]++;
}

void reportSleep(unsigned int time)
{
	sleepTime += time;
}

void reportCycle(unsigned int time)
{
	cycleTime += time;
}

void reportClose(void)
{
	int i;
	for (i = 0; i<I_SIZEOF; i++)
	{
		fprintf(repf, "Instruction,%s,%d\n", instrNameArray[i], instrCount[i]);
	}
	fprintf(repf, "Running time, ,%d\n", cycleTime);
	fprintf(repf, "Sleep time, ,%d\n", sleepTime);
	fprintf(repf, "Pipeline flush, Flush,%d\n", pipeFlush);

	fclose(repf);
}

void reportReset(void)
{
	memset(instrCount, 0, sizeof(instrCount));
	cycleTime = 0;
	sleepTime = 0;
	pipeFlush = 0;
}

void reportPipeFlush(void)
{
	pipeFlush++;
}

void reportParallel(void)
{
	parallelEx++;
}
