/*
 * vcd.c
 *
 *  Created on: Oct 8, 2015
 *      Author: heso
 */

#include "vcd.h" //glob.h included
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include "report.h"
#include <sys/stat.h>
#include <sys/types.h>

static FILE *vcdf;
static long int vcdTime;
static long int clockTime;
static long int timerTime;
static int timerPeriod;
unsigned int clockCycle;
int cpuClock = 1;

void vcdInit(char const* testName)
{
	//-------------------------------------------------------------------------------
	//Print the time to differentiate different runs.
	time_t current_time;
	struct tm * time_info;
	char timeString[10]; // space for "HH:MM:SS\0"
	char dateTimeString[25]; //space for "DAY MON DD HH:MM:SS YYYY\0"

	time(&current_time);
	time_info = localtime(&current_time);

	strftime(timeString, sizeof(timeString), "%j-%H-%M", time_info);
	strftime(dateTimeString, sizeof(dateTimeString), "%c",time_info);

	//-------------------------------------------------------------------------------
	//Create  filename

	//With time indicated in filename for multiple runs
	char* folder = "simFiles";
	char* name = "instrSimulator";
	char* extension = ".vcd";

//	Timestamp in filename removed
//	char fileSpec[strlen(name) + strlen(testName) + strlen(timeString) + strlen(extension) + 1];
//	snprintf(fileSpec, sizeof(fileSpec), "%s%s%s", name, testName, timeString, extension);

	char fileSpec[strlen(folder) + 1 + strlen(name) + strlen(testName) + strlen(extension) + 1];
	snprintf(fileSpec, sizeof(fileSpec), "%s/%s%s%s", folder, name, testName, extension);

	mkdir(folder, S_IRWXU | S_IRWXG | S_IRWXO);

	vcdf = fopen(fileSpec, "w");
	if (vcdf == NULL)
		printf("Error opening file.\n");

	//-------------------------------------------------------------------------------
//Initialize vcd-file
	fprintf(vcdf,
			"$date\n%s\n", dateTimeString
			);
	fputs(
			"$end\n"
			"$version\n"
			"   Generated by simulator.c, an instruction set simulator.\n"
			"$end\n"
			"$comment\n"
			"   .\n"
			"$end\n"
			"$timescale 1ns $end\n"
			"$scope module logic $end\n"
			"$var wire 8 ' OE_N $end\n"
			"$var wire 8 & IE $end\n"
			"$var wire 8 ( IN $end\n"
			"$var wire 8 ) OUT $end\n"
			"$var reg 8 reg0 regFile[0] $end\n"
			"$var reg 8 reg1 regFile[1] $end\n"
			"$var reg 8 reg2 regFile[2] $end\n"
			"$var reg 8 reg3 regFile[3] $end\n"
			"$var reg 8 txR txReg $end\n"
			"$var reg 8 rxR rxReg $end\n"
			"$var reg 8 ctrlR ctrlReg $end\n"
			"$var reg 1 slp sleep $end\n"
			"$var reg 1 ckCpu ckCpu $end\n"
			"$upscope $end\n"
			"$enddefinitions $end\n"
			"$dumpvars\n"
			"#0\n"
			"bxxxxxxxx '\n"
			"bxxxxxxxx &\n"
			"bxxxxxxxx (\n"
			"bxxxxxxxx )\n"
			"bxxxxxxxx reg0\n"
			"bxxxxxxxx reg1\n"
			"bxxxxxxxx reg2\n"
			"bxxxxxxxx reg3\n"
			"bxxxxxxxx txR\n"
			"bxxxxxxxx rxR\n"
			"bxxxxxxxx ctrlR\n"
			"0slp\n"
			"1ckCpu\n"
			"$end\n"
			,vcdf);
}

void vcdIo(io_t padCtrl, unsigned int value)
{
	char buffer[33];//32 bits and \0
	char *twoByte = buffer +25; //Final 9 bits of buffer
	int2Bins(value, buffer, 33);

	switch (padCtrl){
	case OUT:
		fprintf(vcdf, "b%s )\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case IN:
		fprintf(vcdf, "b%s (\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case IE:
		fprintf(vcdf, "b%s &\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case OE_N:
		fprintf(vcdf, "b%s '\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	default:
		fprintf(stderr, "Error in vcd.c: trying to write NON-existing pad control.");
		break;
	}
}


void vcdReg(unsigned int reg, unsigned int value)
{
	char buffer[33];//32 bits and \0
	char *twoByte = buffer + 25; //Final 9 chars of buffer
	int2Bins(value, buffer, 33);

	switch (reg){
	case R0:
		fprintf(vcdf, "b%s reg0\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case R1:
		fprintf(vcdf, "b%s reg1\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case R2:
		fprintf(vcdf, "b%s reg2\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case R3:
		fprintf(vcdf, "b%s reg3\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case RX_REG:
		fprintf(vcdf, "b%s rxR\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case TX_REG:
		fprintf(vcdf, "b%s txR\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	case CTRL_REG:
		fprintf(vcdf, "b%s ctrlR\n", twoByte); //Only final 9 chars
		vcdWait(1);
		break;
	default:
		fprintf(stderr, "Error in vcd.c: trying to write NON-existing register.");
		break;
	}
}

void vcdInReg(unsigned int value)
{
	char buffer[33];//32 bits and \0
	char *twoByte = buffer + 25; //Final 9 chars of buffer
	int2Bins(value, buffer, 33);

	fprintf(vcdf, "b%s rxR\n", twoByte); //Only final 9 chars
	vcdWait(1);
}
//================================================================================
/* Should only be used to wait for short periods to make waveform more readable.
 * I.e. that one things happens after another, not all at the same time. */
void vcdWait(unsigned int value)
{
	vcdTime += value;
	fprintf(vcdf, "#%ld\n", vcdTime);
}

//================================================================================
void vcdWaitCycle(void)
{
	//Cycle cpuClock
	clockTime += CPU_CK_TIME/2;
	cpuClock = !cpuClock;
	fprintf(vcdf, "#%ld\n", clockTime);
	fprintf(vcdf, "%dckCpu\n", cpuClock);
	clockTime += CPU_CK_TIME/2;
	cpuClock = !cpuClock;
	fprintf(vcdf, "#%ld\n", clockTime);
	fprintf(vcdf, "%dckCpu\n", cpuClock);

	clockCycle += 1;
	reportCycle(CPU_CK_TIME);

	if(vcdTime <= clockTime)
	{
		vcdTime = clockTime;
	}
	else
	{
		fprintf(stderr, "Error in vcd.c: vcdTime is larger than cycle time\n");
	}
}

void vcdStartTimer(unsigned int newPeriod)
{
	timerPeriod = newPeriod;
	timerTime = vcdTime;
}

void vcdWaitTimer(void)
{
	timerTime += timerPeriod;
	fprintf(vcdf, "1slp\n");
	if(vcdTime <= clockTime && clockTime <= timerTime)
	{
		reportSleep(timerTime-clockTime);
		vcdTime = timerTime;
		clockTime = timerTime;
		fprintf(vcdf, "#%ld\n", vcdTime);
	}
	else
	{
		fprintf(stderr, "Error in vcd.c: vcdTime: %ld  or clockTime: %ld is larger than timerTime: %ld\n", vcdTime, clockTime, timerTime);
	}
	fprintf(vcdf, "0slp\n");
}

void vcdClose(void)
{
	fclose(vcdf);
}

char *int2Bins(int value, char *buffer, int bufSize) {
    buffer += (bufSize - 1);

    int i;
    for (i = 31; i >= 0; i--) {
        *buffer-- = (value & 1) + '0';

        value >>= 1;
    }

    buffer[33] = '\0'; //End string
    return buffer;
}

void vcdPipeFlush(void)
{
	int i;
	for (i = 0; i < FLUSH_PENALTY; i++)
		vcdWaitCycle();
}

void vcdReset(void)
{
	vcdTime	    = 0;
	clockTime	= 0;
	timerTime	= 0;
	timerPeriod = 0;
	clockCycle  = 0;
	cpuClock 	= 1;
}
