/*
 * i2cMTest.cpp
 *
 *  Created on: Nov 10, 2015
 *      Author: heso
 */

extern "C"
{
#include "../tb/asmIncludes.h"
#include "../tb/ioMap.h"
#include "../tb/regFile.h"
#include "../tb/glob.h"
}

#include "CppUTest/TestHarness.h"
#include <string.h>

TEST_GROUP(i2cMTest)
{

	void setup()
	{
		resetSim(); //reset ioArray, regFile, report, vcd
		ioMapInitAllVal(IN, 0xC0);

	}
	void teardown()
	{
		//This closes files opened with initSim.
		//Each test should have its unique initSim("testname") in order to store to separate files.
		closeSim();
	}
};
//15
TEST(i2cMTest, checkStart)
{
	//Init
	initSim("I2cMCheckStart");
	i2cMInit();

	regWrite(TX_REG, 0xAA); //Address 55 and write

	//Used if multimaster checking is on
//	setInArray(0xAA, MSB, SDA, 8, 0); //AA

	setInArray(0, MSB, SDA, 1, 0); //addr ack
	setInArray(0, MSB, SDA, 1, 1); //write ack

	//Run i2cM
	i2cM(1);

	//SDA and SCL was high, then SDA asserted followed by SCL
	LONGS_EQUAL(0x2, ioMap_GetData(OE_N, MSB, SDA, 2, 0)); //SDA 1->0
	LONGS_EQUAL(0x2, ioMap_GetData(OE_N, MSB, SCL, 2, 1)); //##1 SCL 1->0
}

TEST(i2cMTest, sendDataByte)
{
	//Init
	initSim("I2cMSendDataByte");
	i2cMInit();

	regWrite(TX_REG, 0xBC); // 6F and write

	setInArray(0, MSB, SDA, 1, 0); //addr ack
	setInArray(0, MSB, SDA, 1, 1); //write ack

//Used if multimaster checking is on
//	setInArray(0xBE, MSB, SDA, 8, 0); //BE + ack
//	setInArray(0x000, MSB, SDA, 9, 9); //00 + ack

	//Run i2cM
	i2cM(1);

	LONGS_EQUAL(0xBC, ioMap_GetDataPosedge(OE_N, MSB, SDA, SCL, 8, 1)); //BE
	LONGS_EQUAL(0x00, ioMap_GetDataPosedge(OE_N, MSB, SDA, SCL, 8, 30)); //data
}

TEST(i2cMTest, recDataByte)
{
	//Init
	initSim("I2cMRecDataByte");
	i2cMInit();

	regWrite(TX_REG, 0x59); //2C and read
	setInArray(0, MSB, SDA, 1, 0); //Ack
	setInArray(0xAA, MSB, SDA, 8, 1);


//Used if multimaster checking is on
//	setInArray(0x58, MSB, SDA, 8, 0);

	//Run i2cM
	i2cM(1);

	LONGS_EQUAL(0x59, ioMap_GetDataPosedge(OE_N, MSB, SDA, SCL, 8, 1)); //BE
	LONGS_EQUAL(0xAA, regRead(RX_REG)); //
	LONGS_EQUAL(0x1, ioMap_GetDataPosedge(OE_N, MSB, SDA, SCL, 1, 9+8)); //Check master ack
}

TEST(i2cMTest, checkStop)
{
	//Init
	initSim("I2cMCheckStop");
	i2cMInit();

	regWrite(TX_REG, 0x55); //Address 2A and read

	setInArray(0, MSB, SDA, 1, 0); //ack

//Used if multimaster checking is on
//setInArray(0x55, MSB, SDA, 8, 0); //55
//setInArray(0x000, MSB, SDA, 9, 9); //00 + ack

	//Run i2cM
	i2cM(1);

	//SDA and SCL was high, then SDA asserted followed by SCL
	LONGS_EQUAL(0x1, ioMap_GetData(OE_N, MSB, SCL, 2, 50)); //SCL 0->1
	LONGS_EQUAL(0x1, ioMap_GetData(OE_N, MSB, SDA, 2, 51)); //##1 SDA 0->1

}
/*
TEST(i2cMTest, repStart)
{
	initSim("I2cMRepStart");
	i2cMInit();

	regWrite(TX_REG, 0x6A); //Addr & Write
	setInArray(0, MSB, SDA, 1, 0); //Ack address
	setInArray(0, MSB, SDA, 1, 1); //Ack 1st byte
	setInArray(0, MSB, SDA, 1, 2); //Ack 2nd byte

	i2cM(2);

	LONGS_EQUAL(0x6A, ioMap_GetDataPosedge(OE_N, MSB, SDA, SCL, 8, 1));
	LONGS_EQUAL(0x00, ioMap_GetDataPosedge(OE_N, MSB, SDA, SCL, 8, 30));
	LONGS_EQUAL(0x1, ioMap_GetData(OE_N, MSB, SCL, 2, 58)); //SCL 0->1
	LONGS_EQUAL(0x2, ioMap_GetData(OE_N, MSB, SDA, 2, 59)); //SDA 1->0
	LONGS_EQUAL(0x00, ioMap_GetDataPosedge(OE_N, MSB, SDA, SCL, 8, 60));
}
*/

TEST(i2cMTest, longTransfer)
{
	initSim("I2cMLongTransfer");
	i2cMInit();

	regWrite(TX_REG, 0x4D); //Read

	setInArray(0, MSB, SDA, 1, 0); //Ack
	setInArray(0xAA, MSB, SDA, 8, 1);
	setInArray(0xBB, MSB, SDA, 8, 9);
	setInArray(0xCC, MSB, SDA, 8, 17);
	setInArray(0xDD, MSB, SDA, 8, 25);
	setInArray(0xEE, MSB, SDA, 8, 33);

	i2cM(5);
	//Can only check final byte
	LONGS_EQUAL(0xEE, regRead(RX_REG));
}
