/*
 * spiTest.cpp
 *
 *  Created on: Oct 22, 2015
 *      Author: heso
 */

extern "C"
{
#include "../tb/asmIncludes.h"
#include "../tb/ioMap.h"
#include "../tb/regFile.h"
#include "../tb/glob.h"
}

#include "CppUTest/TestHarness.h"
#include <string.h>

TEST_GROUP(spiMTest)
{

	void setup()
	{
		resetSim(); //reset ioArray, regFile, report, vcd
	}
	void teardown()
	{
		//This closes files opened with initSim.
		//Each test should have its unique initSim("testname") in order to store to separate files.
		closeSim();
	}
};
//15
TEST(spiMTest, sendDataByte)
{
	//Init
	initSim("SpiMSendDataByte");
	spiMInit();
	regWrite(TX_REG, 0xBE);
	//Run spiM
	spiM(1);

	LONGS_EQUAL(0xBE, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2));

}

TEST(spiMTest, chipSelected)
{
	//Init
	initSim("SpiMchipSelected");
	spiMInit();
	regWrite(TX_REG, 0x55);
	//Run spiM
	spiM(1);

	LONGS_EQUAL(0x55, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2));
	LONGS_EQUAL(0x00, ioMap_GetDataPosedge(OUT, MSB, CSn, SCKO, 8, 1));

}

TEST(spiMTest, sendAndReceiveByte)
{
	//Init
	initSim("SpiMSendAndReceiveByte");
	spiMInit();
	regWrite(TX_REG, 0xDC);
	setInArray(0xAC, MSB, MISO, 8, 0);
	//Run spiM
	spiM(1);

	LONGS_EQUAL(0xDC, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 0));
	//LONGS_EQUAL(0xAC, regRead(RX_REG)); //This is not valid after two sends. Should perhaps buffer this reg.

}

//TODO this test will fail as long as it is not possible to write address first and read data after
TEST(spiMTest, sendAdrRecByte)
{
	//Init
	initSim("SpiMSendAdrRecByte");
	spiMInit();
	regWrite(TX_REG, 0x33);
	setInArray(0x77, MSB, MISO, 8, 8);
	//Run spiM
	spiM(2);

	LONGS_EQUAL(0x33, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 0));
	LONGS_EQUAL(0x00, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2));
	LONGS_EQUAL(0x77, regRead(RX_REG));

}

TEST(spiMTest, byteReady)
{
	initSim("SpiMByteReady");
	spiMInit();
	regWrite(TX_REG, 0xAA);
	setInArray(0x55, MSB, MISO, 8, 0);
	//Run spiM
	spiM(1);

	LONGS_EQUAL(0xAA, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 0));
	LONGS_EQUAL(0x55, regRead(RX_REG));
	LONGS_EQUAL(1<<SPI_BYTE_READY, regRead(CTRL_REG) & 1<<SPI_BYTE_READY);
}

TEST(spiMTest, threeTransfers)
{
	initSim("SpiMThreeTransfers");
	spiMInit();

	setInArray(0x2C, MSB, MISO, 8, 0);
	setInArray(0x5E, MSB, MISO, 8, 8);
	setInArray(0x82, MSB, MISO, 8, 16);

	spiMStart();

	regWrite(TX_REG, 0x54);
	spiMTransferByte();
	LONGS_EQUAL(0x2C, regRead(RX_REG));

	regWrite(TX_REG, 0xD7);
	spiMTransferByte();
	LONGS_EQUAL(0x5E, regRead(RX_REG));

	regWrite(TX_REG, 0xB8);
	spiMTransferByte();
	LONGS_EQUAL(0x82, regRead(RX_REG));

	LONGS_EQUAL(0x54, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2));
	LONGS_EQUAL(0xD7, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2));
	LONGS_EQUAL(0xB8, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*2));

	spiMEnd();
}

TEST(spiMTest, longTransfer)
{
	initSim("SpiMLongTransfer");
	spiMInit();

	setInArray(0xE5, MSB, MISO, 8, 0);
	setInArray(0xFF, MSB, MISO, 8, 8);
	setInArray(0x00, MSB, MISO, 8, 16);
	setInArray(0x22, MSB, MISO, 8, 24);
	setInArray(0x80, MSB, MISO, 8, 32);
	setInArray(0x01, MSB, MISO, 8, 40);
	setInArray(0xDD, MSB, MISO, 8, 48);
	setInArray(0x33, MSB, MISO, 8, 56);
	setInArray(0x42, MSB, MISO, 8, 64);

	spiMStart();

	regWrite(TX_REG, 0xFF);
	spiMTransferByte();
	LONGS_EQUAL(0xE5, regRead(RX_REG));

	regWrite(TX_REG, 0x00);
	spiMTransferByte();
	LONGS_EQUAL(0xFF, regRead(RX_REG));

	regWrite(TX_REG, 0x80);
	spiMTransferByte();
	LONGS_EQUAL(0x00, regRead(RX_REG));

	regWrite(TX_REG, 0x01);
	spiMTransferByte();
	LONGS_EQUAL(0x22, regRead(RX_REG));

	regWrite(TX_REG, 0xDD);
	spiMTransferByte();
	LONGS_EQUAL(0x80, regRead(RX_REG));

	regWrite(TX_REG, 0x33);
	spiMTransferByte();
	LONGS_EQUAL(0x01, regRead(RX_REG));

	regWrite(TX_REG, 0x42);
	spiMTransferByte();
	LONGS_EQUAL(0xDD, regRead(RX_REG));

	regWrite(TX_REG, 0x55);
	spiMTransferByte();
	LONGS_EQUAL(0x33, regRead(RX_REG));

	regWrite(TX_REG, 0xAA);
	spiMTransferByte();
	LONGS_EQUAL(0x42, regRead(RX_REG));

	regWrite(TX_REG, 0xAA);
	spiMTransferByte();
	LONGS_EQUAL(0x00, regRead(RX_REG));

	regWrite(TX_REG, 0xAA);
	spiMTransferByte();
	LONGS_EQUAL(0x00, regRead(RX_REG));

	regWrite(TX_REG, 0xAA);
	spiMTransferByte();
	LONGS_EQUAL(0x00, regRead(RX_REG));

	spiMEnd();

	LONGS_EQUAL(0xFF, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2));
	LONGS_EQUAL(0x00, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*1));
	LONGS_EQUAL(0x80, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*2));
	LONGS_EQUAL(0x01, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*3));
	LONGS_EQUAL(0xDD, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*4));
	LONGS_EQUAL(0x33, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*5));
	LONGS_EQUAL(0x42, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*6));
	LONGS_EQUAL(0x55, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*7));
	LONGS_EQUAL(0xAA, ioMap_GetDataPosedge(OUT, MSB, MOSI, SCKO, 8, 2+8*2*8));
}
