/*
 * uartTest.cpp
 *
 *  Created on: Oct 26, 2015
 *      Author: heso
 */

extern "C"
{
#include "../tb/asmIncludes.h"
#include "../tb/ioMap.h"
#include "../tb/regFile.h"
#include "../tb/glob.h"
}

#include "CppUTest/TestHarness.h"
#include <string.h>

TEST_GROUP(uartFullTest)
{

	void setup()
	{
		resetSim(); //reset ioArray, regFile, report, vcd

	}
	void teardown()
	{
		//This closes files opened with initSim.
		//Each test should have its unique initSim in order to store to separate files.
		closeSim();
	}
};

TEST(uartFullTest, sendByte)
{
	initSim("UartFullSendByte");
	setInArray(0xFFFF, LSB, RXD, 16, 0); //Idle
	regWrite(TX_REG, 0xAA);
	debugRegWriteBit(CTRL_REG, TXSTART, 1);

	uartSetup();
	uartFull(10);

	LONGS_EQUAL(0xAA, ioMap_GetData(OUT, LSB, TXD, 8, 2));
}

TEST(uartFullTest, checkStartEndBit)
{
	initSim("UartFullCheckStartEndBit");
	setInArray(0xFFFF, LSB, RXD, 16, 0); //Idle
	regWrite(TX_REG, 0x0F);
	debugRegWriteBit(CTRL_REG, TXSTART, 1);

	uartSetup();
	uartFull(10);

	LONGS_EQUAL(0x1, ioMap_GetData(OUT, MSB, TXD, 2, 1)); //Check start bit + LSB
	LONGS_EQUAL(0x1, ioMap_GetData(OUT, MSB, TXD, 2, 9)); //Check stop bit + MSB

}

TEST(uartFullTest, longIdle)
{
	initSim("UartFullLongIdle");
	setInArray(0x1, LSB, RXD, 2, 0); //Start bit  //10
	setInArray(0x2B, LSB, RXD, 8, 2); //Data	  //01010101
	setInArray(0xFFFFFFFFFFFFFFFF, LSB, RXD, 64, 10); //Stop	  //1111

	uartSetup();
	uartFull(20);
	regWrite(TX_REG, 0xEA);
	debugRegWriteBit(CTRL_REG, TXSTART, 1);
	uartFull(48);

	LONGS_EQUAL(0x2B, regRead(RX_REG));
	LONGS_EQUAL(0xEA, ioMap_GetData(OUT, LSB, TXD, 8, 2));
}

TEST(uartFullTest, recByte)
{
	initSim("UartFullRecByte");
	setInArray(0x1, LSB, RXD, 2, 0); //Start bit  //10
	setInArray(0xAA, LSB, RXD, 8, 2); //Data	  //01010101
	setInArray(0xFFFF, LSB, RXD, 16, 10); //Stop	  //1111

	uartSetup();
	uartFull(14);

	LONGS_EQUAL(0xAA, regRead(RX_REG));
}


TEST(uartFullTest, longTransfer)
{

	initSim("UartFullLongTransfer");
	setInArray(0x7, LSB, RXD, 4, 0); //Start bit
	setInArray(0xAA, LSB, RXD, 8, 4); //Data
	setInArray(0x7F, LSB, RXD, 8, 12); //Stop and start
	setInArray(0xFF, LSB, RXD, 8, 20); //Data
	setInArray(0x0F, LSB, RXD, 5, 28); //Stop and start
	setInArray(0x00, LSB, RXD, 8, 33); //Data
	setInArray(0x0FFFF, LSB, RXD, 17, 41); //Stop and start
	setInArray(0x55, LSB, RXD, 8, 58); //Data
	setInArray(0xFFFF, LSB, RXD, 16, 66); //Stop
	//Runs until 70

	//-----TEST: Delayed start
	uartSetup();
	uartFull(6);

	regWrite(TX_REG, 0xFF);
	debugRegWriteBit(CTRL_REG, TXSTART, 1);

	uartFull(11); //17
	LONGS_EQUAL(0xAA, regRead(RX_REG));
	//----------------------

	//-----TEST: send empty & start send before ended
	regWrite(TX_REG, 0xAA);
	debugRegWriteBit(CTRL_REG, TXSTART, 1);
	uartFull(11); //28
	debugRegWriteBit(CTRL_REG, TXSTART, 1);
	LONGS_EQUAL(0xFF, regRead(RX_REG));
	uartFull(9); //37
	//-----------------------

	//----TEST: Back to back send
	regWrite(TX_REG, 0x55);
	debugRegWriteBit(CTRL_REG, TXSTART, 1);

	uartFull(10); //47
	LONGS_EQUAL(0x00, regRead(RX_REG));

	regWrite(TX_REG, 0xFF);
	debugRegWriteBit(CTRL_REG, TXSTART, 1);
	uartFull(10); //57
	//---------------------

	//----TEST: Delayed send command

	regWrite(TX_REG, 0x44);

	uartFull(2); //59

	debugRegWriteBit(CTRL_REG, TXSTART, 1);

	uartFull(14); //73
	LONGS_EQUAL(0x55, regRead(RX_REG));

	//------------------------------


	LONGS_EQUAL(0xFF, ioMap_GetData(OUT, LSB, TXD, 8, 2));
	LONGS_EQUAL(0xAA, ioMap_GetData(OUT, LSB, TXD, 8, 12));
	LONGS_EQUAL(0x00, ioMap_GetData(OUT, LSB, TXD, 8, 22));
	LONGS_EQUAL(0x55, ioMap_GetData(OUT, LSB, TXD, 8, 32));
	LONGS_EQUAL(0xFF, ioMap_GetData(OUT, LSB, TXD, 8, 42));
	LONGS_EQUAL(0x44, ioMap_GetData(OUT, LSB, TXD, 8, 52));

}
